@@ -416,9 +416,11 @@ int  wc_AesGcmEncrypt(Aes* aes, byte* out,
     \brief This function decrypts the input cipher text, held in the buffer
     in, and stores the resulting message text in the output buffer out.
     It also checks the input authentication vector, authIn, against the
-    supplied authentication tag, authTag.
+    supplied authentication tag, authTag.  If a nonzero error code is returned,
+    the output data is undefined.  However, callers must unconditionally zeroize
+    the output buffer to guard against leakage of cleartext data.
 
-    \return 0 On successfully decrypting the input message
+    \return 0 On successfully decrypting and authenticating the input message
     \return AES_GCM_AUTH_E If the authentication tag does not match the
     supplied authentication code vector, authTag.
 
@@ -609,8 +611,9 @@ int  wc_AesCcmEncrypt(Aes* aes, byte* out,
     \brief This function decrypts the input cipher text, in, into
     the output buffer, out, using CCM (Counter with CBC-MAC). It
     subsequently calculates the authorization tag, authTag, from the
-    authIn input. If the authorization tag is invalid, it sets the
-    output buffer to zero and returns the error: AES_CCM_AUTH_E.
+    authIn input.  If a nonzero error code is returned, the output data is
+    undefined.  However, callers must unconditionally zeroize the output buffer
+    to guard against leakage of cleartext data.
 
     \return 0 On successfully decrypting the input message
     \return AES_CCM_AUTH_E If the authentication tag does not match the
@@ -668,13 +671,13 @@ int  wc_AesCcmDecrypt(Aes* aes, byte* out,
 
     \param aes   AES keys for encrypt/decrypt process
     \param heap  heap hint to use for memory. Can be NULL
-    \param devId id to use with async crypto. Can be 0
+    \param devId ID to use with crypto callbacks or async hardware. Set to INVALID_DEVID (-2) if not used
 
     _Example_
     \code
     XtsAes aes;
 
-    if(wc_AesXtsInit(&aes, NULL, 0) != 0)
+    if(wc_AesXtsInit(&aes, NULL, INVALID_DEVID) != 0)
     {
         // Handle error
     }
@@ -749,13 +752,13 @@ int wc_AesXtsSetKeyNoInit(XtsAes* aes, const byte* key,
                  i.e. 32 for a 16 byte key.
     \param dir   direction, either AES_ENCRYPTION or AES_DECRYPTION
     \param heap  heap hint to use for memory. Can be NULL
-    \param devId id to use with async crypto. Can be 0
+    \param devId ID to use with crypto callbacks or async hardware. Set to INVALID_DEVID (-2) if not used
 
     _Example_
     \code
     XtsAes aes;
 
-    if(wc_AesXtsSetKey(&aes, key, sizeof(key), AES_ENCRYPTION, NULL, 0) != 0)
+    if(wc_AesXtsSetKey(&aes, key, sizeof(key), AES_ENCRYPTION, NULL, INVALID_DEVID) != 0)
     {
         // Handle error
     }
@@ -974,7 +977,7 @@ int wc_AesXtsFree(XtsAes* aes);
 
     \param aes aes structure in to initialize
     \param heap heap hint to use for malloc / free if needed
-    \param devId ID to use with async hardware
+    \param devId ID to use with crypto callbacks or async hardware. Set to INVALID_DEVID (-2) if not used
 
     _Example_
     \code
@@ -1134,7 +1137,9 @@ int wc_AesSivEncrypt(const byte* key, word32 keySz, const byte* assoc,
 /*!
     \ingroup AES
     \brief This function performs SIV (synthetic initialization vector)
-    decryption as described in RFC 5297.
+    decryption as described in RFC 5297.  If a nonzero error code is returned,
+    the output data is undefined.  However, callers must unconditionally zeroize
+    the output buffer to guard against leakage of cleartext data.
 
     \return 0 On successful decryption.
     \return BAD_FUNC_ARG If key, SIV, or output buffer are NULL. Also returned
@@ -1248,7 +1253,10 @@ WOLFSSL_API int  wc_AesEaxEncryptAuth(const byte* key, word32 keySz, byte* out,
     \brief This function performs AES EAX decryption and authentication as
     described in "EAX: A Conventional Authenticated-Encryption Mode"
     (https://eprint.iacr.org/2003/069). It is a "one-shot" API that performs
-    all decryption and authentication operations in one function call.
+    all decryption and authentication operations in one function call.  If a
+    nonzero error code is returned, the output data is undefined.
+    However, callers must unconditionally zeroize the output buffer to guard
+    against leakage of cleartext data.
 
     \return 0 on successful decryption
     \return BAD_FUNC_ARG if input or output buffers are NULL. Also returned
@@ -1455,7 +1463,7 @@ WOLFSSL_API int  wc_AesEaxEncryptUpdate(AesEax* eax, byte* out,
     This argument should be NULL if not used
     \param authInSz size in bytes of the input authentication data
 
-    
+
     _Example_
     \code
     AesEax eax;
@@ -1571,8 +1579,8 @@ WOLFSSL_API int  wc_AesEaxAuthDataUpdate(AesEax* eax,
 
 /*!
     \ingroup AES
-    \brief This function finalizes the encrypt AEAD operation, producing an auth 
-    tag over the current authentication stream. \c eax must have been previously 
+    \brief This function finalizes the encrypt AEAD operation, producing an auth
+    tag over the current authentication stream. \c eax must have been previously
     initialized with a call to \ref wc_AesEaxInit. When done using the \c AesEax
     context structure, make sure to free it using \ref wc_AesEaxFree.
 
@@ -1632,10 +1640,10 @@ WOLFSSL_API int wc_AesEaxEncryptFinal(AesEax* eax,
 
 /*!
     \ingroup AES
-    \brief This function finalizes the decrypt AEAD operation, finalizing the 
+    \brief This function finalizes the decrypt AEAD operation, finalizing the
     auth tag computation and checking it for validity against the user supplied
-    tag. \c eax must have been previously initialized with a call to 
-    \ref wc_AesEaxInit. When done using the \c AesEax context structure, make 
+    tag. \c eax must have been previously initialized with a call to
+    \ref wc_AesEaxInit. When done using the \c AesEax context structure, make
     sure to free it using \ref wc_AesEaxFree.
 
     \return 0 if data is authenticated successfully
@@ -1725,5 +1733,244 @@ WOLFSSL_API int wc_AesEaxDecryptFinal(AesEax* eax,
 */
 WOLFSSL_API int wc_AesEaxFree(AesEax* eax);
 
+/*!
+    \ingroup AES
+    \brief This function performs AES encryption using Ciphertext Stealing (CTS)
+    mode. It is a one-shot API that handles all operations in a single call.
 
+    \return 0 on successful encryption.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \return other negative error codes for encryption failures.
 
+    \param key pointer to the AES key used for encryption.
+    \param keySz size of the AES key in bytes (16, 24, or 32 bytes).
+    \param[out] out buffer to hold the encrypted ciphertext. Must be at least
+    the size of the input.
+    \param in pointer to the plaintext input data to encrypt.
+    \param inSz size of the plaintext input data in bytes.
+    \param iv pointer to the initialization vector (IV) used for encryption.
+    Must be 16 bytes.
+
+    _Example_
+    \code
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte plaintext[] = { 0x01, 0x02, 0x03, 0x04, 0x05 };
+        byte ciphertext[sizeof(plaintext)];
+
+        int ret = wc_AesCtsEncrypt(key, sizeof(key), ciphertext, plaintext,
+            sizeof(plaintext), iv);
+        if (ret != 0) {
+        // handle encryption error
+    }
+    \endcode
+
+    \sa wc_AesCtsDecrypt
+*/
+int wc_AesCtsEncrypt(const byte* key, word32 keySz, byte* out,
+                     const byte* in, word32 inSz,
+                     const byte* iv);
+
+/*!
+    \ingroup AES
+    \brief This function performs AES encryption using Ciphertext Stealing (CTS)
+     mode. It is a one-shot API that handles all operations in a single call.
+
+    \return 0 on successful encryption.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \return other negative error codes for encryption failures.
+
+    \param key pointer to the AES key used for encryption.
+    \param keySz size of the AES key in bytes (16, 24, or 32 bytes).
+    \param[out] out buffer to hold the encrypted ciphertext. Must be at least
+                 the same size as the input plaintext.
+    \param in pointer to the plaintext input data to encrypt.
+    \param inSz size of the plaintext input data in bytes.
+    \param iv pointer to the initialization vector (IV) used for encryption.
+             Must be 16 bytes.
+    _Example_
+    \code
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte plaintext[] = { 0x01, 0x02, 0x03, 0x04, 0x05 };
+        byte ciphertext[sizeof(plaintext)];
+        int ret = wc_AesCtsEncrypt(key, sizeof(key), ciphertext, plaintext,
+                                   sizeof(plaintext), iv);
+        if (ret != 0) {
+            // handle encryption error
+        }
+    \endcode
+    \sa wc_AesCtsDecrypt
+*/
+int wc_AesCtsEncrypt(const byte* key, word32 keySz, byte* out,
+                     const byte* in, word32 inSz,
+                     const byte* iv);
+
+/*!
+    \ingroup AES
+    \brief This function performs AES decryption using Ciphertext Stealing (CTS) mode.
+           It is a one-shot API that handles all operations in a single call.
+    \return 0 on successful decryption.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \return other negative error codes for decryption failures.
+    \param key pointer to the AES key used for decryption.
+    \param keySz size of the AES key in bytes (16, 24, or 32 bytes).
+    \param[out] out buffer to hold the decrypted plaintext. Must be at least
+                 the same size as the input ciphertext.
+    \param in pointer to the ciphertext input data to decrypt.
+    \param inSz size of the ciphertext input data in bytes.
+    \param iv pointer to the initialization vector (IV) used for decryption.
+             Must be 16 bytes.
+    _Example_
+    \code
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte ciphertext[] = { 0x01, 0x02, 0x03, 0x04, 0x05 };
+        byte plaintext[sizeof(ciphertext)];
+        int ret = wc_AesCtsDecrypt(key, sizeof(key), plaintext, ciphertext,
+                                   sizeof(ciphertext), iv);
+        if (ret != 0) {
+            // handle decryption error
+        }
+    \endcode
+    \sa wc_AesCtsEncrypt
+*/
+int wc_AesCtsDecrypt(const byte* key, word32 keySz, byte* out,
+                     const byte* in, word32 inSz,
+                     const byte* iv);
+
+/*!
+    \ingroup AES
+    \brief This function performs an update step of the AES CTS encryption.
+           It processes a chunk of plaintext and stores intermediate data.
+    \return 0 on successful processing.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \param aes pointer to the Aes structure holding the context of the operation.
+    \param[out] out buffer to hold the encrypted ciphertext. Must be large enough
+                 to store the output from this update step.
+    \param[out] outSz size in bytes of the output data written to the \c out buffer.
+                     On input, it should contain the maximum number of bytes that can
+                     be written to the \c out buffer.
+    \param in pointer to the plaintext input data to encrypt.
+    \param inSz size of the plaintext input data in bytes.
+    _Example_
+    \code
+        Aes aes;
+        wc_AesInit(&aes, NULL, INVALID_DEVID);
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte plaintext[] = { ... };
+        byte ciphertext[sizeof(plaintext)];
+        word32 outSz = sizeof(ciphertext);
+        wc_AesSetKey(&aes, key, sizeof(key), iv, AES_ENCRYPTION);
+        int ret = wc_AesCtsEncryptUpdate(&aes, ciphertext, &outSz, plaintext, sizeof(plaintext));
+        if (ret != 0) {
+            // handle error
+        }
+        wc_AesFree(&aes);
+    \endcode
+    \sa wc_AesCtsDecryptUpdate
+*/
+int wc_AesCtsEncryptUpdate(Aes* aes, byte* out, word32* outSz,
+                           const byte* in, word32 inSz);
+
+/*!
+    \ingroup AES
+    \brief This function finalizes the AES CTS encryption operation.
+           It processes any remaining plaintext and completes the encryption.
+    \return 0 on successful encryption completion.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \param aes pointer to the Aes structure holding the context of the operation.
+    \param[out] out buffer to hold the final encrypted ciphertext. Must be large
+                 enough to store any remaining ciphertext from this final step.
+    \param[out] outSz size in bytes of the output data written to the \c out buffer.
+                     On input, it should contain the maximum number of bytes that can
+                     be written to the \c out buffer.
+    _Example_
+    \code
+        Aes aes;
+        wc_AesInit(&aes, NULL, INVALID_DEVID);
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte plaintext[] = { ... };
+        byte ciphertext[sizeof(plaintext)];
+        word32 outSz = sizeof(ciphertext);
+        wc_AesSetKey(&aes, key, sizeof(key), iv, AES_ENCRYPTION);
+        // Perform any required update steps using wc_AesCtsEncryptUpdate
+        int ret = wc_AesCtsEncryptFinal(&aes, ciphertext, &outSz);
+        if (ret != 0) {
+            // handle error
+        }
+        wc_AesFree(&aes);
+    \endcode
+    \sa wc_AesCtsDecryptFinal
+*/
+int wc_AesCtsEncryptFinal(Aes* aes, byte* out, word32* outSz);
+
+/*!
+    \ingroup AES
+    \brief This function performs an update step of the AES CTS decryption.
+           It processes a chunk of ciphertext and stores intermediate data.
+    \return 0 on successful processing.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \param aes pointer to the Aes structure holding the context of the operation.
+    \param[out] out buffer to hold the decrypted plaintext. Must be large enough
+                 to store the output from this update step.
+    \param[out] outSz size in bytes of the output data written to the \c out buffer.
+                     On input, it should contain the maximum number of bytes that can
+                     be written to the \c out buffer.
+    \param in pointer to the ciphertext input data to decrypt.
+    \param inSz size of the ciphertext input data in bytes.
+    _Example_
+    \code
+        Aes aes;
+        wc_AesInit(&aes, NULL, INVALID_DEVID);
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte ciphertext[] = { ... };
+        byte plaintext[sizeof(ciphertext)];
+        word32 outSz = sizeof(plaintext);
+        wc_AesSetKey(&aes, key, sizeof(key), iv, AES_DECRYPTION);
+        int ret = wc_AesCtsDecryptUpdate(&aes, plaintext, &outSz, ciphertext, sizeof(ciphertext));
+        if (ret != 0) {
+            // handle error
+        }
+        wc_AesFree(&aes);
+    \endcode
+    \sa wc_AesCtsEncryptUpdate
+*/
+int wc_AesCtsDecryptUpdate(Aes* aes, byte* out, word32* outSz,
+                           const byte* in, word32 inSz);
+
+/*!
+    \ingroup AES
+    \brief This function finalizes the AES CTS decryption operation.
+           It processes any remaining ciphertext and completes the decryption.
+    \return 0 on successful decryption completion.
+    \return BAD_FUNC_ARG if input arguments are invalid.
+    \param aes pointer to the Aes structure holding the context of the operation.
+    \param[out] out buffer to hold the final decrypted plaintext. Must be large
+                 enough to store any remaining plaintext from this final step.
+    \param[out] outSz size in bytes of the output data written to the \c out buffer.
+                     On input, it should contain the maximum number of bytes that can
+                     be written to the \c out buffer.
+    _Example_
+    \code
+        Aes aes;
+        wc_AesInit(&aes, NULL, INVALID_DEVID);
+        byte key[16] = { 0 };
+        byte iv[16] = { 0 };
+        byte ciphertext[] = { ... };
+        byte plaintext[sizeof(ciphertext)];
+        word32 outSz = sizeof(plaintext);
+        wc_AesSetKey(&aes, key, sizeof(key), iv, AES_DECRYPTION);
+        // Perform any required update steps using wc_AesCtsDecryptUpdate
+        int ret = wc_AesCtsDecryptFinal(&aes, plaintext, &outSz);
+        if (ret != 0) {
+            // handle error
+        }
+        wc_AesFree(&aes);
+    \endcode
+    \sa wc_AesCtsEncryptFinal
+*/
+int wc_AesCtsDecryptFinal(Aes* aes, byte* out, word32* outSz);
