@@ -35,7 +35,7 @@
     \sa Base64_Encode
     \sa Base16_Decode
 */
-WOLFSSL_API int Base64_Decode(const byte* in, word32 inLen, byte* out,
+int Base64_Decode(const byte* in, word32 inLen, byte* out,
                                word32* outLen);
 
 /*!
@@ -75,7 +75,7 @@ WOLFSSL_API int Base64_Decode(const byte* in, word32 inLen, byte* out,
     \sa Base64_EncodeEsc
     \sa Base64_Decode
 */
-WOLFSSL_API
+
 int Base64_Encode(const byte* in, word32 inLen, byte* out,
                                   word32* outLen);
 
@@ -157,7 +157,7 @@ int Base64_EncodeEsc(const byte* in, word32 inLen, byte* out,
     \sa Base64_Encode
     \sa Base64_Decode
 */
-WOLFSSL_API
+
 int Base64_Encode_NoNl(const byte* in, word32 inLen, byte* out,
                                   word32* outLen);
 
@@ -196,7 +196,7 @@ int Base64_Encode_NoNl(const byte* in, word32 inLen, byte* out,
     \sa Base64_Decode
     \sa Base16_Encode
 */
-WOLFSSL_API
+
 int Base16_Decode(const byte* in, word32 inLen, byte* out, word32* outLen);
 
 /*!
@@ -229,5 +229,5 @@ int Base16_Decode(const byte* in, word32 inLen, byte* out, word32* outLen);
     \sa Base64_Decode
     \sa Base16_Decode
 */
-WOLFSSL_API
+
 int Base16_Encode(const byte* in, word32 inLen, byte* out, word32* outLen);
