@@ -17,6 +17,8 @@
         <li>\ref Base_Encoding</li>
         <li>\ref Compression</li>
         <li>\ref Error</li>
+        <li>\ref IoTSafe</li>
+        <li>\ref PSA</li>
         <li>\ref Keys</li>
         <li>\ref Logging</li>
         <li>\ref Math</li>
@@ -32,6 +34,8 @@
         <li>\ref Camellia</li>
         <li>\ref ChaCha</li>
         <li>\ref ChaCha20Poly1305</li>
+        <li>\ref CMAC</li>
+        <li>\ref Crypto Callbacks</li>
         <li>\ref Curve25519</li>
         <li>\ref Curve448</li>
         <li>\ref DSA</li>
@@ -39,9 +43,9 @@
         <li>\ref ECC</li>
         <li>\ref ED25519</li>
         <li>\ref ED448</li>
-        <li>\ref HC128</li>
+        <li>\ref ECCSI</li>
+        <li>\ref SAKKE</li>
         <li>\ref HMAC</li>
-        <li>\ref IDEA</li>
         <li>\ref MD2</li>
         <li>\ref MD4</li>
         <li>\ref MD5</li>
@@ -49,10 +53,25 @@
         <li>\ref PKCS7</li>
         <li>\ref PKCS11</li>
         <li>\ref Poly1305</li>
-        <li>\ref Rabbit</li>
         <li>\ref RIPEMD</li>
         <li>\ref RSA</li>
         <li>\ref SHA</li>
+        <li>\ref SipHash</li>
+        <li>\ref SrtpKdf</li>
         <li>\ref SRP</li>
     </ul>
 */
+/*!
+    \page ECCSI ECCSI API Reference
+    - \ref ECCSI_Overview
+    - \ref ECCSI_Setup
+    - \ref ECCSI_Operations
+*/
+/*!
+    \page SAKKE SAKKE API Reference
+    - \ref SAKKE_Overview
+    - \ref SAKKE_Setup
+    - \ref SAKKE_RSK
+    - \ref SAKKE_Operations
+*/
+
