@@ -1,12 +1,12 @@
 /* eccsi.c
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -19,13 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
  */
 
-
-
-#ifdef HAVE_CONFIG_H
-    #include <config.h>
-#endif
-
-#include <wolfssl/wolfcrypt/settings.h>
+#include <wolfssl/wolfcrypt/libwolfssl_sources.h>
 
 #ifdef NO_INLINE
     #include <wolfssl/wolfcrypt/misc.h>
@@ -36,13 +30,26 @@
 
 #ifdef WOLFCRYPT_HAVE_ECCSI
 
-#include <wolfssl/wolfcrypt/error-crypt.h>
 #include <wolfssl/wolfcrypt/eccsi.h>
 #include <wolfssl/wolfcrypt/asn_public.h>
 #ifdef WOLFSSL_HAVE_SP_ECC
     #include <wolfssl/wolfcrypt/sp.h>
 #endif
 
+#if defined(WOLFSSL_LINUXKM) && !defined(WOLFSSL_SP_ASM)
+    /* force off unneeded vector register save/restore. */
+    #undef SAVE_VECTOR_REGISTERS
+    #define SAVE_VECTOR_REGISTERS(fail_clause) WC_DO_NOTHING
+    #undef RESTORE_VECTOR_REGISTERS
+    #define RESTORE_VECTOR_REGISTERS() WC_DO_NOTHING
+#endif
+
+#ifndef WOLFSSL_HAVE_ECC_KEY_GET_PRIV
+    /* FIPS build has replaced ecc.h. */
+    #define wc_ecc_key_get_priv(key) (&((key)->k))
+    #define WOLFSSL_HAVE_ECC_KEY_GET_PRIV
+#endif
+
 /**
  * Initialize the components of the ECCSI key and use the specified curve.
  *
@@ -155,6 +162,7 @@ void wc_FreeEccsiKey(EccsiKey* key)
         wc_ecc_del_point_h(key->pvt, key->heap);
         wc_ecc_free(&key->pubkey);
         wc_ecc_free(&key->ecc);
+        XMEMSET(key, 0, sizeof(*key));
     }
 }
 
@@ -383,10 +391,12 @@ static int eccsi_compute_hs(EccsiKey* key, enum wc_HashType hashType,
     word32 dataSz = 0;
     int idx = wc_ecc_get_curve_idx(key->ecc.dp->id);
     ecc_point* kpak = &key->ecc.pubkey;
+    int hash_inited = 0;
 
     /* HS = hash( G | KPAK | ID | PVT ) */
     err = wc_HashInit_ex(&key->hash, hashType, key->heap, INVALID_DEVID);
     if (err == 0) {
+        hash_inited = 1;
         /* Base Point - G */
         dataSz = sizeof(key->data);
         err = eccsi_encode_base(key, key->data, &dataSz);
@@ -426,6 +436,10 @@ static int eccsi_compute_hs(EccsiKey* key, enum wc_HashType hashType,
         *hashSz = (byte)wc_HashGetDigestSize(hashType);
     }
 
+    if (hash_inited) {
+        (void)wc_HashFree(&key->hash, hashType);
+    }
+
     return err;
 }
 
@@ -495,7 +509,7 @@ static int eccsi_encode_point(ecc_point* point, word32 size, byte* data,
 
     if (data == NULL) {
         *sz = size * 2 + !raw;
-        err = LENGTH_ONLY_E;
+        err = WC_NO_ERR_TRACE(LENGTH_ONLY_E);
     }
     if ((err == 0) && (*sz < size * 2 + !raw)) {
         err = BUFFER_E;
@@ -508,12 +522,12 @@ static int eccsi_encode_point(ecc_point* point, word32 size, byte* data,
         }
 
         /* Write out the point's x ordinate into key size bytes. */
-        err = mp_to_unsigned_bin_len(point->x, data, size);
+        err = mp_to_unsigned_bin_len(point->x, data, (int)size);
     }
     if (err == 0) {
         data += size;
         /* Write out the point's y ordinate into key size bytes. */
-        err = mp_to_unsigned_bin_len(point->y, data, size);
+        err = mp_to_unsigned_bin_len(point->y, data, (int)size);
     }
     if (err == 0) {
         *sz = size * 2 + !raw;
@@ -587,7 +601,8 @@ static int eccsi_encode_key(EccsiKey* key, byte* data)
     word32 sz = (word32)key->ecc.dp->size * 2;
 
     /* Write out the secret value into key size bytes. */
-    err = mp_to_unsigned_bin_len(&key->ecc.k, data, key->ecc.dp->size);
+    err = mp_to_unsigned_bin_len(wc_ecc_key_get_priv(&key->ecc), data,
+        key->ecc.dp->size);
     if (err == 0) {
         data += key->ecc.dp->size;
         /* Write the public key. */
@@ -632,14 +647,14 @@ int wc_ExportEccsiKey(EccsiKey* key, byte* data, word32* sz)
 
     if (err == 0) {
         if (data == NULL) {
-            *sz = key->ecc.dp->size * 3;
-            err = LENGTH_ONLY_E;
+            *sz = (word32)(key->ecc.dp->size * 3);
+            err = WC_NO_ERR_TRACE(LENGTH_ONLY_E);
         }
         else if (*sz < (word32)key->ecc.dp->size * 3) {
             err = BUFFER_E;
         }
         else {
-            *sz = key->ecc.dp->size * 3;
+            *sz = (word32)(key->ecc.dp->size * 3);
         }
     }
     if (err == 0) {
@@ -669,12 +684,13 @@ static int eccsi_decode_key(EccsiKey* key, const byte* data)
     int err;
 
     /* Read the secret value from key size bytes. */
-    err = mp_read_unsigned_bin(&key->ecc.k, data, key->ecc.dp->size);
+    err = mp_read_unsigned_bin(wc_ecc_key_get_priv(&key->ecc), data,
+        (word32)key->ecc.dp->size);
     if (err == 0) {
         data += key->ecc.dp->size;
         /* Read public key. */
         err = eccsi_decode_point(&key->ecc.pubkey, (word32)key->ecc.dp->size,
-                data, key->ecc.dp->size * 2);
+                data, (word32)(key->ecc.dp->size * 2));
     }
 
     return err;
@@ -753,18 +769,19 @@ int wc_ExportEccsiPrivateKey(EccsiKey* key, byte* data, word32* sz)
 
     if (err == 0) {
         if (data == NULL) {
-            *sz = key->ecc.dp->size;
-            err = LENGTH_ONLY_E;
+            *sz = (word32)key->ecc.dp->size;
+            err = WC_NO_ERR_TRACE(LENGTH_ONLY_E);
         }
         else if (*sz < (word32)key->ecc.dp->size) {
             err = BUFFER_E;
         }
         else {
-            *sz = key->ecc.dp->size;
+            *sz = (word32)key->ecc.dp->size;
         }
     }
     if (err == 0) {
-        err = mp_to_unsigned_bin_len(&key->ecc.k, data, key->ecc.dp->size);
+        err = mp_to_unsigned_bin_len(wc_ecc_key_get_priv(&key->ecc), data,
+            key->ecc.dp->size);
     }
 
     return err;
@@ -797,7 +814,8 @@ int wc_ImportEccsiPrivateKey(EccsiKey* key, const byte* data, word32 sz)
     }
 
     if (err == 0) {
-        err = mp_read_unsigned_bin(&key->ecc.k, data, key->ecc.dp->size);
+        err = mp_read_unsigned_bin(wc_ecc_key_get_priv(&key->ecc), data,
+            (word32)key->ecc.dp->size);
     }
 
     return err;
@@ -900,18 +918,20 @@ static int eccsi_make_pair(EccsiKey* key, WC_RNG* rng,
             err = mp_read_unsigned_bin(ssk, key->data, hashSz);
         }
         if (err == 0) {
-            err = mp_mulmod(ssk, &key->pubkey.k, &key->params.order, ssk);
+            err = mp_mulmod(ssk, wc_ecc_key_get_priv(&key->pubkey),
+                &key->params.order, ssk);
         }
         if (err == 0) {
-            err = mp_addmod(ssk, &key->ecc.k, &key->params.order, ssk);
+            err = mp_addmod(ssk, wc_ecc_key_get_priv(&key->ecc),
+                &key->params.order, ssk);
         }
     }
     while ((err == 0) && (mp_iszero(ssk) ||
-            (mp_cmp(ssk, &key->ecc.k) == MP_EQ)));
+            (mp_cmp(ssk, wc_ecc_key_get_priv(&key->ecc)) == MP_EQ)));
     /* Step 5: ensure SSK and HS are non-zero (code lines above) */
 
     /* Step 6: Copy out SSK (done during calc) and PVT. Erase v */
-    mp_forcezero(&key->pubkey.k);
+    mp_forcezero(wc_ecc_key_get_priv(&key->pubkey));
 
     return err;
 }
@@ -988,8 +1008,8 @@ int wc_EncodeEccsiPair(const EccsiKey* key, mp_int* ssk, ecc_point* pvt,
     }
 
     if ((err == 0) && (data == NULL)) {
-        *sz = key->ecc.dp->size * 3;
-        err = LENGTH_ONLY_E;
+        *sz = (word32)(key->ecc.dp->size * 3);
+        err = WC_NO_ERR_TRACE(LENGTH_ONLY_E);
     }
     if ((err == 0) && (*sz < (word32)(key->ecc.dp->size * 3))) {
         err = BUFFER_E;
@@ -1009,7 +1029,7 @@ int wc_EncodeEccsiPair(const EccsiKey* key, mp_int* ssk, ecc_point* pvt,
         err = mp_to_unsigned_bin_len(pvt->y, data, key->ecc.dp->size);
     }
     if (err == 0) {
-        *sz = key->ecc.dp->size * 3;
+        *sz = (word32)(key->ecc.dp->size * 3);
     }
 
     return err;
@@ -1049,14 +1069,14 @@ int wc_EncodeEccsiSsk(const EccsiKey* key, mp_int* ssk, byte* data, word32* sz)
 
     if (err == 0) {
         if (data == NULL) {
-            *sz = key->ecc.dp->size;
-            err = LENGTH_ONLY_E;
+            *sz = (word32)key->ecc.dp->size;
+            err = WC_NO_ERR_TRACE(LENGTH_ONLY_E);
         }
         else if (*sz < (word32)key->ecc.dp->size) {
             err = BUFFER_E;
         }
         else {
-            *sz = key->ecc.dp->size;
+            *sz = (word32)key->ecc.dp->size;
         }
     }
     if (err == 0) {
@@ -1095,7 +1115,7 @@ int wc_DecodeEccsiSsk(const EccsiKey* key, const byte* data, word32 sz,
     }
 
     if (err == 0) {
-        err = mp_read_unsigned_bin(ssk, data, key->ecc.dp->size);
+        err = mp_read_unsigned_bin(ssk, data, (word32)key->ecc.dp->size);
     }
 
     return err;
@@ -1172,17 +1192,17 @@ int wc_DecodeEccsiPair(const EccsiKey* key, const byte* data, word32 sz,
 
     if (err == 0) {
         /* Read the SSK value from key size bytes. */
-        err = mp_read_unsigned_bin(ssk, data, key->ecc.dp->size);
+        err = mp_read_unsigned_bin(ssk, data, (word32)key->ecc.dp->size);
     }
     if (err == 0) {
         data += key->ecc.dp->size;
         /* Read the PVT's x value from key size bytes. */
-        err = mp_read_unsigned_bin(pvt->x, data, key->ecc.dp->size);
+        err = mp_read_unsigned_bin(pvt->x, data, (word32)key->ecc.dp->size);
     }
     if (err == 0) {
         data += key->ecc.dp->size;
         /* Read the PVT's y value from key size bytes. */
-        err = mp_read_unsigned_bin(pvt->y, data, key->ecc.dp->size);
+        err = mp_read_unsigned_bin(pvt->y, data, (word32)key->ecc.dp->size);
     }
     if (err == 0) {
         err = mp_set(pvt->z, 1);
@@ -1256,7 +1276,7 @@ int wc_DecodeEccsiPvtFromSig(const EccsiKey* key, const byte* sig, word32 sz,
     }
 
     if (err == 0) {
-        word32 rSz = key->ecc.dp->size * 2;
+        word32 rSz = (word32)(key->ecc.dp->size * 2);
         err = eccsi_decode_point(pvt, (word32)key->ecc.dp->size, sig + rSz,
                 sz - rSz);
     }
@@ -1331,15 +1351,13 @@ static int eccsi_mulmod_base_add(EccsiKey* key, const mp_int* n,
 {
     int err = 0;
 
-#ifdef WOLFSSL_HAVE_SP_ECC
-#ifndef WOLFSSL_SP_NO_256
+#if defined(WOLFSSL_HAVE_SP_ECC) && !defined(WOLFSSL_SP_NO_256)
     if ((key->ecc.idx != ECC_CUSTOM_IDX) &&
             (ecc_sets[key->ecc.idx].id == ECC_SECP256R1)) {
         err = sp_ecc_mulmod_base_add_256(n, a, 1, res, map, key->heap);
     }
     else
 #endif
-#endif
 #ifndef WOLFSSL_SP_MATH
     {
         EccsiKeyParams* params = &key->params;
@@ -1358,7 +1376,12 @@ static int eccsi_mulmod_base_add(EccsiKey* key, const mp_int* n,
     {
         err = NOT_COMPILED_IN;
     }
+    (void)key;
+    (void)n;
+    (void)a;
+    (void)res;
     (void)mp;
+    (void)map;
 #endif
 
     return err;
@@ -1382,14 +1405,12 @@ static int eccsi_mulmod_point(EccsiKey* key, const mp_int* n, ecc_point* point,
 {
     int err;
 
-#ifdef WOLFSSL_HAVE_SP_ECC
-#ifndef WOLFSSL_SP_NO_256
+#if defined(WOLFSSL_HAVE_SP_ECC) && !defined(WOLFSSL_SP_NO_256)
     if ((key->ecc.idx != ECC_CUSTOM_IDX) &&
             (ecc_sets[key->ecc.idx].id == ECC_SECP256R1)) {
         err = sp_ecc_mulmod_256(n, point, res, map, key->heap);
     }
     else
-#endif
 #endif
     {
         EccsiKeyParams* params = &key->params;
@@ -1418,9 +1439,8 @@ static int eccsi_mulmod_point(EccsiKey* key, const mp_int* n, ecc_point* point,
 static int eccsi_mulmod_point_add(EccsiKey* key, const mp_int* n,
         ecc_point* point, ecc_point* a, ecc_point* res, mp_digit mp, int map)
 {
-#ifdef WOLFSSL_HAVE_SP_ECC
-#ifndef WOLFSSL_SP_NO_256
-    int err = NOT_COMPILED_IN;
+#if defined(WOLFSSL_HAVE_SP_ECC) && !defined(WOLFSSL_SP_NO_256)
+    int err = WC_NO_ERR_TRACE(NOT_COMPILED_IN);
 
     if ((key->ecc.idx != ECC_CUSTOM_IDX) &&
             (ecc_sets[key->ecc.idx].id == ECC_SECP256R1)) {
@@ -1430,7 +1450,6 @@ static int eccsi_mulmod_point_add(EccsiKey* key, const mp_int* n,
     (void)mp;
 
     return err;
-#endif
 #else
     int err;
     EccsiKeyParams* params = &key->params;
@@ -1611,6 +1630,7 @@ int wc_ValidateEccsiPvt(EccsiKey* key, const ecc_point* pvt, int* valid)
  * @param  [out]  hashSz    Length of hash data in bytes.
  * @return  0 on success.
  * @return  BAD_FUNC_ARG when key, id, pvt, hash or hashSz is NULL.
+ * @return  BAD_FUNC_ARG when hash size doesn't match curve size.
  * @return  BAD_STATE_E when public key not set.
  * @return  MEMORY_E when dynamic memory allocation fails.
  * @return  Other -ve value when an internal operation fails.
@@ -1619,6 +1639,8 @@ int wc_HashEccsiId(EccsiKey* key, enum wc_HashType hashType, const byte* id,
         word32 idSz, ecc_point* pvt, byte* hash, byte* hashSz)
 {
     int err = 0;
+    int dgstSz = -1;
+    int curveSz = -1;
 
     if ((key == NULL) || (id == NULL) || (pvt == NULL) || (hash == NULL) ||
             (hashSz == NULL)) {
@@ -1628,6 +1650,22 @@ int wc_HashEccsiId(EccsiKey* key, enum wc_HashType hashType, const byte* id,
             (key->ecc.type != ECC_PUBLICKEY)) {
         err = BAD_STATE_E;
     }
+    /* Ensure digest output size matches curve size (RFC 6507 4.1). */
+    if (err == 0) {
+        dgstSz = wc_HashGetDigestSize(hashType);
+        if (dgstSz < 0) {
+            err = dgstSz;
+        }
+    }
+    if (err == 0) {
+        curveSz = wc_ecc_get_curve_size_from_id(key->ecc.dp->id);
+        if (curveSz < 0) {
+            err = curveSz;
+        }
+    }
+    if ((err == 0) && (dgstSz != curveSz)) {
+        err = BAD_FUNC_ARG;
+    }
     /* Load the curve parameters for operations */
     if (err == 0) {
        err = eccsi_load_ecc_params(key);
@@ -1773,16 +1811,18 @@ static int eccsi_compute_he(EccsiKey* key, enum wc_HashType hashType,
         mp_int* r, const byte* msg, word32 msgSz, byte* he, word32* heSz)
 {
     int err = 0;
-    word32 dataSz = key->ecc.dp->size;
+    word32 dataSz = (word32)key->ecc.dp->size;
+    int hash_inited = 0;
 
     /* HE = hash( HS | r | M ) */
     err = wc_HashInit_ex(&key->hash, hashType, key->heap, INVALID_DEVID);
     if (err == 0) {
+        hash_inited = 1;
         /* HS */
         err = wc_HashUpdate(&key->hash, hashType, key->idHash, key->idHashSz);
     }
     if (err == 0) {
-        err = mp_to_unsigned_bin_len(r, key->data, dataSz);
+        err = mp_to_unsigned_bin_len(r, key->data, (int)dataSz);
     }
     if (err == 0) {
         /* r */
@@ -1796,7 +1836,11 @@ static int eccsi_compute_he(EccsiKey* key, enum wc_HashType hashType,
         err = wc_HashFinal(&key->hash, hashType, he);
     }
     if (err == 0) {
-        *heSz = wc_HashGetDigestSize(hashType);
+        *heSz = (word32)wc_HashGetDigestSize(hashType);
+    }
+
+    if (hash_inited) {
+        (void)wc_HashFree(&key->hash, hashType);
     }
 
     return err;
@@ -1816,14 +1860,14 @@ static int eccsi_encode_sig(const EccsiKey* key, mp_int* r, mp_int* s,
         byte* sig, word32* sigSz)
 {
     int err;
-    word32 sz = key->ecc.dp->size;
+    word32 sz = (word32)key->ecc.dp->size;
 
-    err = mp_to_unsigned_bin_len(r, sig, sz);
+    err = mp_to_unsigned_bin_len(r, sig, (int)sz);
     if (err == 0) {
-        err = mp_to_unsigned_bin_len(s, sig + sz, sz);
+        err = mp_to_unsigned_bin_len(s, sig + sz, (int)sz);
     }
     if (err == 0) {
-        *sigSz = key->ecc.dp->size * 2 + 1;
+        *sigSz = (word32)(key->ecc.dp->size * 2 + 1);
         err = wc_ecc_export_point_der(wc_ecc_get_curve_idx(key->ecc.dp->id),
                  key->pvt, sig + sz * 2, sigSz);
     }
@@ -1854,7 +1898,7 @@ static int eccsi_gen_sig(EccsiKey* key, WC_RNG* rng, enum wc_HashType hashType,
         const byte* msg, word32 msgSz, mp_int* r, mp_int* s)
 {
     int err = 0;
-    word32 sz = key->ecc.dp->size;
+    int sz = key->ecc.dp->size;
     word32 heSz = 0;
     const mp_int* jx = NULL;
     mp_int* he = &key->tmp;
@@ -1946,10 +1990,10 @@ int wc_SignEccsiHash(EccsiKey* key, WC_RNG* rng, enum wc_HashType hashType,
     }
 
     if (err == 0)  {
-        sz = key->ecc.dp->size;
+        sz = (word32)key->ecc.dp->size;
         if (sig == NULL) {
             *sigSz = sz * 4 + 1;
-            err = LENGTH_ONLY_E;
+            err = WC_NO_ERR_TRACE(LENGTH_ONLY_E);
         }
     }
     if ((err == 0) && (*sigSz < sz * 4 + 1)) {
@@ -1973,14 +2017,14 @@ int wc_SignEccsiHash(EccsiKey* key, WC_RNG* rng, enum wc_HashType hashType,
         err = mp_invmod(s, &key->params.order, s);
     }
     if (err == 0) {
-        j  = &key->pubkey.k;
+        j = wc_ecc_key_get_priv(&key->pubkey);
         err = mp_mulmod(s, j, &key->params.order, s);
     }
     if (err == 0) {
         mp_forcezero(j);
 
         /* Step 6: s = s' fitted */
-        err = eccsi_fit_to_octets(s, &key->params.order, sz, s);
+        err = eccsi_fit_to_octets(s, &key->params.order, (int)sz, s);
     }
 
     /* Step 7: Output Signature = ( r | s | PVT ) */
@@ -2006,7 +2050,7 @@ static int eccsi_decode_sig_s(const EccsiKey* key, const byte* sig,
         word32 sigSz, mp_int* s)
 {
     int err = 0;
-    word32 sz = key->ecc.dp->size;
+    word32 sz = (word32)key->ecc.dp->size;
 
     if (sigSz != sz * 4 + 1) {
         err = BAD_FUNC_ARG;
@@ -2035,7 +2079,7 @@ static int eccsi_decode_sig_r_pvt(const EccsiKey* key, const byte* sig,
         word32 sigSz, mp_int* r, ecc_point* pvt)
 {
     int err = 0;
-    word32 sz = key->ecc.dp->size;
+    word32 sz = (word32)key->ecc.dp->size;
 
     if (sigSz != sz * 4 + 1) {
         err = BAD_FUNC_ARG;
@@ -2183,7 +2227,7 @@ int wc_VerifyEccsiHash(EccsiKey* key, enum wc_HashType hashType,
     SAVE_VECTOR_REGISTERS(return _svr_ret;);
 
     /* Decode the signature into components. */
-    r = &key->pubkey.k;
+    r = wc_ecc_key_get_priv(&key->pubkey);
     pvt = &key->pubkey.pubkey;
     err = eccsi_decode_sig_r_pvt(key, sig, sigSz, r, pvt);
 
