@@ -1,12 +1,12 @@
 /* fe_448.c
  *
- * Copyright (C) 2006-2022 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -24,11 +24,7 @@
  * Reworked for curve448 by Sean Parkinson.
  */
 
-#ifdef HAVE_CONFIG_H
-    #include <config.h>
-#endif
-
-#include <wolfssl/wolfcrypt/settings.h>
+#include <wolfssl/wolfcrypt/libwolfssl_sources.h>
 
 #if defined(HAVE_CURVE448) || defined(HAVE_ED448)
 
@@ -167,7 +163,7 @@ void fe448_sub(word8* r, const word8* a, const word8* b)
     }
 }
 
-/* Mulitply a field element by 39081. r = (39081 * a) mod (2^448 - 2^224 - 1)
+/* Multiply a field element by 39081. r = (39081 * a) mod (2^448 - 2^224 - 1)
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -192,7 +188,7 @@ void fe448_mul39081(word8* r, const word8* a)
     }
 }
 
-/* Mulitply two field elements. r = (a * b) mod (2^448 - 2^224 - 1)
+/* Multiply two field elements. r = (a * b) mod (2^448 - 2^224 - 1)
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -358,14 +354,13 @@ int curve448(byte* r, const byte* n, const byte* a)
     word8 t1[56];
     int i;
     unsigned int swap;
-    unsigned int b;
 
     fe448_copy(x1, a);
     fe448_copy(x3, a);
 
     swap = 0;
     for (i = 447; i >= 0; --i) {
-        b = (n[i >> 3] >> (i & 7)) & 1;
+        unsigned int b = (n[i >> 3] >> (i & 7)) & 1;
         swap ^= b;
         fe448_cswap(x2, x3, swap);
         fe448_cswap(z2, z3, swap);
@@ -449,7 +444,7 @@ void fe448_neg(word8* r, const word8* a)
 }
 
 /* Raise field element to (p-3) / 4: 2^446 - 2^222 - 1
- * Used for calcualting y-ordinate from x-ordinate for Ed448.
+ * Used for calculating y-ordinate from x-ordinate for Ed448.
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to exponentiate.
@@ -607,65 +602,65 @@ void fe448_to_bytes(unsigned char* b, const sword64* a)
     o = (in5  >> 56); in6  += o; t = o << 56; in5  -= (sword64)t;
     o = (in6  >> 56); in7  += o; t = o << 56; in6  -= (sword64)t;
     o = (in7  >> 56); in0  += o;
-                    in4  += o; t = o << 56; in7  -= (sword64)t;
+                      in4  += o; t = o << 56; in7  -= (sword64)t;
 
     /* Output as bytes */
-    b[ 0] = (in0  >>  0);
-    b[ 1] = (in0  >>  8);
-    b[ 2] = (in0  >> 16);
-    b[ 3] = (in0  >> 24);
-    b[ 4] = (in0  >> 32);
-    b[ 5] = (in0  >> 40);
-    b[ 6] = (in0  >> 48);
-    b[ 7] = (in1  >>  0);
-    b[ 8] = (in1  >>  8);
-    b[ 9] = (in1  >> 16);
-    b[10] = (in1  >> 24);
-    b[11] = (in1  >> 32);
-    b[12] = (in1  >> 40);
-    b[13] = (in1  >> 48);
-    b[14] = (in2  >>  0);
-    b[15] = (in2  >>  8);
-    b[16] = (in2  >> 16);
-    b[17] = (in2  >> 24);
-    b[18] = (in2  >> 32);
-    b[19] = (in2  >> 40);
-    b[20] = (in2  >> 48);
-    b[21] = (in3  >>  0);
-    b[22] = (in3  >>  8);
-    b[23] = (in3  >> 16);
-    b[24] = (in3  >> 24);
-    b[25] = (in3  >> 32);
-    b[26] = (in3  >> 40);
-    b[27] = (in3  >> 48);
-    b[28] = (in4  >>  0);
-    b[29] = (in4  >>  8);
-    b[30] = (in4  >> 16);
-    b[31] = (in4  >> 24);
-    b[32] = (in4  >> 32);
-    b[33] = (in4  >> 40);
-    b[34] = (in4  >> 48);
-    b[35] = (in5  >>  0);
-    b[36] = (in5  >>  8);
-    b[37] = (in5  >> 16);
-    b[38] = (in5  >> 24);
-    b[39] = (in5  >> 32);
-    b[40] = (in5  >> 40);
-    b[41] = (in5  >> 48);
-    b[42] = (in6  >>  0);
-    b[43] = (in6  >>  8);
-    b[44] = (in6  >> 16);
-    b[45] = (in6  >> 24);
-    b[46] = (in6  >> 32);
-    b[47] = (in6  >> 40);
-    b[48] = (in6  >> 48);
-    b[49] = (in7  >>  0);
-    b[50] = (in7  >>  8);
-    b[51] = (in7  >> 16);
-    b[52] = (in7  >> 24);
-    b[53] = (in7  >> 32);
-    b[54] = (in7  >> 40);
-    b[55] = (in7  >> 48);
+    b[ 0] = (byte)(in0  >>  0);
+    b[ 1] = (byte)(in0  >>  8);
+    b[ 2] = (byte)(in0  >> 16);
+    b[ 3] = (byte)(in0  >> 24);
+    b[ 4] = (byte)(in0  >> 32);
+    b[ 5] = (byte)(in0  >> 40);
+    b[ 6] = (byte)(in0  >> 48);
+    b[ 7] = (byte)(in1  >>  0);
+    b[ 8] = (byte)(in1  >>  8);
+    b[ 9] = (byte)(in1  >> 16);
+    b[10] = (byte)(in1  >> 24);
+    b[11] = (byte)(in1  >> 32);
+    b[12] = (byte)(in1  >> 40);
+    b[13] = (byte)(in1  >> 48);
+    b[14] = (byte)(in2  >>  0);
+    b[15] = (byte)(in2  >>  8);
+    b[16] = (byte)(in2  >> 16);
+    b[17] = (byte)(in2  >> 24);
+    b[18] = (byte)(in2  >> 32);
+    b[19] = (byte)(in2  >> 40);
+    b[20] = (byte)(in2  >> 48);
+    b[21] = (byte)(in3  >>  0);
+    b[22] = (byte)(in3  >>  8);
+    b[23] = (byte)(in3  >> 16);
+    b[24] = (byte)(in3  >> 24);
+    b[25] = (byte)(in3  >> 32);
+    b[26] = (byte)(in3  >> 40);
+    b[27] = (byte)(in3  >> 48);
+    b[28] = (byte)(in4  >>  0);
+    b[29] = (byte)(in4  >>  8);
+    b[30] = (byte)(in4  >> 16);
+    b[31] = (byte)(in4  >> 24);
+    b[32] = (byte)(in4  >> 32);
+    b[33] = (byte)(in4  >> 40);
+    b[34] = (byte)(in4  >> 48);
+    b[35] = (byte)(in5  >>  0);
+    b[36] = (byte)(in5  >>  8);
+    b[37] = (byte)(in5  >> 16);
+    b[38] = (byte)(in5  >> 24);
+    b[39] = (byte)(in5  >> 32);
+    b[40] = (byte)(in5  >> 40);
+    b[41] = (byte)(in5  >> 48);
+    b[42] = (byte)(in6  >>  0);
+    b[43] = (byte)(in6  >>  8);
+    b[44] = (byte)(in6  >> 16);
+    b[45] = (byte)(in6  >> 24);
+    b[46] = (byte)(in6  >> 32);
+    b[47] = (byte)(in6  >> 40);
+    b[48] = (byte)(in6  >> 48);
+    b[49] = (byte)(in7  >>  0);
+    b[50] = (byte)(in7  >>  8);
+    b[51] = (byte)(in7  >> 16);
+    b[52] = (byte)(in7  >> 24);
+    b[53] = (byte)(in7  >> 32);
+    b[54] = (byte)(in7  >> 40);
+    b[55] = (byte)(in7  >> 48);
 }
 
 /* Set the field element to 0.
@@ -789,7 +784,7 @@ void fe448_sub(sword64* r, const sword64* a, const sword64* b)
     r[7] = a[7] - b[7];
 }
 
-/* Mulitply a field element by 39081. r = (39081 * a) mod (2^448 - 2^224 - 1)
+/* Multiply a field element by 39081. r = (39081 * a) mod (2^448 - 2^224 - 1)
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -814,7 +809,7 @@ void fe448_mul39081(sword64* r, const sword64* a)
     o = (sword64)(t5  >> 56); t6  += o; t = (sword128)o << 56; t5  -= t;
     o = (sword64)(t6  >> 56); t7  += o; t = (sword128)o << 56; t6  -= t;
     o = (sword64)(t7  >> 56); t0  += o;
-                   t4  += o; t = (sword128)o << 56; t7  -= t;
+                              t4  += o; t = (sword128)o << 56; t7  -= t;
 
     /* Store */
     r[0] = (sword64)t0;
@@ -827,7 +822,7 @@ void fe448_mul39081(sword64* r, const sword64* a)
     r[7] = (sword64)t7;
 }
 
-/* Mulitply two field elements. r = (a * b) mod (2^448 - 2^224 - 1)
+/* Multiply two field elements. r = (a * b) mod (2^448 - 2^224 - 1)
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -1077,7 +1072,6 @@ int curve448(byte* r, const byte* n, const byte* a)
     sword64 t1[8];
     int i;
     unsigned int swap;
-    unsigned int b;
 
     fe448_from_bytes(x1, a);
     fe448_1(x2);
@@ -1087,10 +1081,10 @@ int curve448(byte* r, const byte* n, const byte* a)
 
     swap = 0;
     for (i = 447; i >= 0; --i) {
-        b = (n[i >> 3] >> (i & 7)) & 1;
+        unsigned int b = (n[i >> 3] >> (i & 7)) & 1;
         swap ^= b;
-        fe448_cswap(x2, x3, swap);
-        fe448_cswap(z2, z3, swap);
+        fe448_cswap(x2, x3, (int)swap);
+        fe448_cswap(z2, z3, (int)swap);
         swap = b;
 
         /* Montgomery Ladder - double and add */
@@ -1174,7 +1168,7 @@ void fe448_neg(sword64* r, const sword64* a)
 }
 
 /* Raise field element to (p-3) / 4: 2^446 - 2^222 - 1
- * Used for calcualting y-ordinate from x-ordinate for Ed448.
+ * Used for calculating y-ordinate from x-ordinate for Ed448.
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to exponentiate.
@@ -1433,65 +1427,65 @@ void fe448_to_bytes(unsigned char* b, const sword32* a)
     o = (in13 >> 28); in14 += o; t = o << 28; in13 -= (sword32)t;
     o = (in14 >> 28); in15 += o; t = o << 28; in14 -= (sword32)t;
     o = (in15 >> 28); in0  += o;
-                    in8  += o; t = o << 28; in15 -= (sword32)t;
+                      in8  += o; t = o << 28; in15 -= (sword32)t;
 
     /* Output as bytes */
-    b[ 0] = (in0  >>  0);
-    b[ 1] = (in0  >>  8);
-    b[ 2] = (in0  >> 16);
-    b[ 3] = (in0  >> 24) + ((in1  >>  0) <<  4);
-    b[ 4] = (in1  >>  4);
-    b[ 5] = (in1  >> 12);
-    b[ 6] = (in1  >> 20);
-    b[ 7] = (in2  >>  0);
-    b[ 8] = (in2  >>  8);
-    b[ 9] = (in2  >> 16);
-    b[10] = (in2  >> 24) + ((in3  >>  0) <<  4);
-    b[11] = (in3  >>  4);
-    b[12] = (in3  >> 12);
-    b[13] = (in3  >> 20);
-    b[14] = (in4  >>  0);
-    b[15] = (in4  >>  8);
-    b[16] = (in4  >> 16);
-    b[17] = (in4  >> 24) + ((in5  >>  0) <<  4);
-    b[18] = (in5  >>  4);
-    b[19] = (in5  >> 12);
-    b[20] = (in5  >> 20);
-    b[21] = (in6  >>  0);
-    b[22] = (in6  >>  8);
-    b[23] = (in6  >> 16);
-    b[24] = (in6  >> 24) + ((in7  >>  0) <<  4);
-    b[25] = (in7  >>  4);
-    b[26] = (in7  >> 12);
-    b[27] = (in7  >> 20);
-    b[28] = (in8  >>  0);
-    b[29] = (in8  >>  8);
-    b[30] = (in8  >> 16);
-    b[31] = (in8  >> 24) + ((in9  >>  0) <<  4);
-    b[32] = (in9  >>  4);
-    b[33] = (in9  >> 12);
-    b[34] = (in9  >> 20);
-    b[35] = (in10 >>  0);
-    b[36] = (in10 >>  8);
-    b[37] = (in10 >> 16);
-    b[38] = (in10 >> 24) + ((in11 >>  0) <<  4);
-    b[39] = (in11 >>  4);
-    b[40] = (in11 >> 12);
-    b[41] = (in11 >> 20);
-    b[42] = (in12 >>  0);
-    b[43] = (in12 >>  8);
-    b[44] = (in12 >> 16);
-    b[45] = (in12 >> 24) + ((in13 >>  0) <<  4);
-    b[46] = (in13 >>  4);
-    b[47] = (in13 >> 12);
-    b[48] = (in13 >> 20);
-    b[49] = (in14 >>  0);
-    b[50] = (in14 >>  8);
-    b[51] = (in14 >> 16);
-    b[52] = (in14 >> 24) + ((in15 >>  0) <<  4);
-    b[53] = (in15 >>  4);
-    b[54] = (in15 >> 12);
-    b[55] = (in15 >> 20);
+    b[ 0] = (byte)(in0  >>  0);
+    b[ 1] = (byte)(in0  >>  8);
+    b[ 2] = (byte)(in0  >> 16);
+    b[ 3] = (byte)((byte)(in0  >> 24) + (byte)((in1  >>  0) <<  4));
+    b[ 4] = (byte)(in1  >>  4);
+    b[ 5] = (byte)(in1  >> 12);
+    b[ 6] = (byte)(in1  >> 20);
+    b[ 7] = (byte)(in2  >>  0);
+    b[ 8] = (byte)(in2  >>  8);
+    b[ 9] = (byte)(in2  >> 16);
+    b[10] = (byte)((byte)(in2  >> 24) + (byte)((in3  >>  0) <<  4));
+    b[11] = (byte)(in3  >>  4);
+    b[12] = (byte)(in3  >> 12);
+    b[13] = (byte)(in3  >> 20);
+    b[14] = (byte)(in4  >>  0);
+    b[15] = (byte)(in4  >>  8);
+    b[16] = (byte)(in4  >> 16);
+    b[17] = (byte)((byte)(in4  >> 24) + (byte)((in5  >>  0) <<  4));
+    b[18] = (byte)(in5  >>  4);
+    b[19] = (byte)(in5  >> 12);
+    b[20] = (byte)(in5  >> 20);
+    b[21] = (byte)(in6  >>  0);
+    b[22] = (byte)(in6  >>  8);
+    b[23] = (byte)(in6  >> 16);
+    b[24] = (byte)((byte)(in6  >> 24) + (byte)((in7  >>  0) <<  4));
+    b[25] = (byte)(in7  >>  4);
+    b[26] = (byte)(in7  >> 12);
+    b[27] = (byte)(in7  >> 20);
+    b[28] = (byte)(in8  >>  0);
+    b[29] = (byte)(in8  >>  8);
+    b[30] = (byte)(in8  >> 16);
+    b[31] = (byte)((byte)(in8  >> 24) + (byte)((in9  >>  0) <<  4));
+    b[32] = (byte)(in9  >>  4);
+    b[33] = (byte)(in9  >> 12);
+    b[34] = (byte)(in9  >> 20);
+    b[35] = (byte)(in10 >>  0);
+    b[36] = (byte)(in10 >>  8);
+    b[37] = (byte)(in10 >> 16);
+    b[38] = (byte)((byte)(in10 >> 24) + (byte)((in11 >>  0) <<  4));
+    b[39] = (byte)(in11 >>  4);
+    b[40] = (byte)(in11 >> 12);
+    b[41] = (byte)(in11 >> 20);
+    b[42] = (byte)(in12 >>  0);
+    b[43] = (byte)(in12 >>  8);
+    b[44] = (byte)(in12 >> 16);
+    b[45] = (byte)((byte)(in12 >> 24) + (byte)((in13 >>  0) <<  4));
+    b[46] = (byte)(in13 >>  4);
+    b[47] = (byte)(in13 >> 12);
+    b[48] = (byte)(in13 >> 20);
+    b[49] = (byte)(in14 >>  0);
+    b[50] = (byte)(in14 >>  8);
+    b[51] = (byte)(in14 >> 16);
+    b[52] = (byte)((byte)(in14 >> 24) + (byte)((in15 >>  0) <<  4));
+    b[53] = (byte)(in15 >>  4);
+    b[54] = (byte)(in15 >> 12);
+    b[55] = (byte)(in15 >> 20);
 }
 
 /* Set the field element to 0.
@@ -1701,7 +1695,7 @@ void fe448_reduce(sword32* a)
     o = a[15] >> 28; a[0]  += (sword32)o;
                      a[8]  += (sword32)o; a[15] -= (sword32)(o << 28);
 }
-/* Mulitply a field element by 39081. r = (39081 * a) mod (2^448 - 2^224 - 1)
+/* Multiply a field element by 39081. r = (39081 * a) mod (2^448 - 2^224 - 1)
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -1742,7 +1736,7 @@ void fe448_mul39081(sword32* r, const sword32* a)
     o = (sword32)(t13 >> 28); t14 += o; t = (sword64)o << 28; t13 -= t;
     o = (sword32)(t14 >> 28); t15 += o; t = (sword64)o << 28; t14 -= t;
     o = (sword32)(t15 >> 28); t0  += o;
-                   t8  += o; t = (sword64)o << 28; t15 -= t;
+                              t8  += o; t = (sword64)o << 28; t15 -= t;
 
     /* Store */
     r[0] = (sword32)t0;
@@ -1763,7 +1757,7 @@ void fe448_mul39081(sword32* r, const sword32* a)
     r[15] = (sword32)t15;
 }
 
-/* Mulitply two field elements. r = a * b
+/* Multiply two field elements. r = a * b
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -1772,6 +1766,8 @@ void fe448_mul39081(sword32* r, const sword32* a)
 static WC_INLINE void fe448_mul_8(sword32* r, const sword32* a, const sword32* b)
 {
     sword64 t;
+    sword64 o;
+    sword64 t15;
     sword64 t0   = (sword64)a[ 0] * b[ 0];
     sword64 t1   = (sword64)a[ 0] * b[ 1];
     sword64 t101 = (sword64)a[ 1] * b[ 0];
@@ -1845,7 +1841,6 @@ static WC_INLINE void fe448_mul_8(sword32* r, const sword32* a, const sword32* b
     t6  += t606;
     t7  += t107; t7  += t207; t7  += t307; t7  += t407; t7  += t507;
     t7  += t607;
-    t7  += t707;
     t8  += t108; t8  += t208; t8  += t308; t8  += t408; t8  += t508;
     t8  += t608;
     t9  += t109; t9  += t209; t9  += t309; t9  += t409; t9  += t509;
@@ -1853,8 +1848,8 @@ static WC_INLINE void fe448_mul_8(sword32* r, const sword32* a, const sword32* b
     t11 += t111; t11 += t211; t11 += t311;
     t12 += t112; t12 += t212;
     t13 += t113;
-    sword64 o = t14 >> 28;
-    sword64 t15 = o;
+    o = t14 >> 28;
+    t15 = o;
     t14 -= o << 28;
     o = (t0  >> 28); t1  += o; t = o << 28; t0  -= t;
     o = (t1  >> 28); t2  += o; t = o << 28; t1  -= t;
@@ -1863,6 +1858,7 @@ static WC_INLINE void fe448_mul_8(sword32* r, const sword32* a, const sword32* b
     o = (t4  >> 28); t5  += o; t = o << 28; t4  -= t;
     o = (t5  >> 28); t6  += o; t = o << 28; t5  -= t;
     o = (t6  >> 28); t7  += o; t = o << 28; t6  -= t;
+    o = (t707>> 28); t8  += o; t = o << 28; t707-= t; t7 += t707;
     o = (t7  >> 28); t8  += o; t = o << 28; t7  -= t;
     o = (t8  >> 28); t9  += o; t = o << 28; t8  -= t;
     o = (t9  >> 28); t10 += o; t = o << 28; t9  -= t;
@@ -1872,7 +1868,7 @@ static WC_INLINE void fe448_mul_8(sword32* r, const sword32* a, const sword32* b
     o = (t13 >> 28); t14 += o; t = o << 28; t13 -= t;
     o = (t14 >> 28); t15 += o; t = o << 28; t14 -= t;
     o = (t15 >> 28); t0  += o;
-                   t8  += o; t = o << 28; t15 -= t;
+                     t8  += o; t = o << 28; t15 -= t;
 
     /* Store */
     r[0] = (sword32)t0;
@@ -1893,7 +1889,7 @@ static WC_INLINE void fe448_mul_8(sword32* r, const sword32* a, const sword32* b
     r[15] = (sword32)t15;
 }
 
-/* Mulitply two field elements. r = (a * b) mod (2^448 - 2^224 - 1)
+/* Multiply two field elements. r = (a * b) mod (2^448 - 2^224 - 1)
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to multiply.
@@ -1994,7 +1990,7 @@ static WC_INLINE void fe448_sqr_8(sword32* r, const sword32* a)
     t4  += t104; t4  += t204;
     t5  += t105; t5  += t205;
     t6  += t106; t6  += t206; t6  += t306;
-    t7  += t107; t7  += t207; t7  += t307;
+    t7  += t107; t7  += t207;
     t8  += t108; t8  += t208; t8  += t308;
     t9  += t109; t9  += t209;
     t10 += t110; t10 += t210;
@@ -2010,6 +2006,7 @@ static WC_INLINE void fe448_sqr_8(sword32* r, const sword32* a)
     o = (t4  >> 28); t5  += o; t = o << 28; t4  -= t;
     o = (t5  >> 28); t6  += o; t = o << 28; t5  -= t;
     o = (t6  >> 28); t7  += o; t = o << 28; t6  -= t;
+    o = (t307>> 28); t8  += o; t = o << 28; t307-= t; t7 += t307;
     o = (t7  >> 28); t8  += o; t = o << 28; t7  -= t;
     o = (t8  >> 28); t9  += o; t = o << 28; t8  -= t;
     o = (t9  >> 28); t10 += o; t = o << 28; t9  -= t;
@@ -2019,7 +2016,7 @@ static WC_INLINE void fe448_sqr_8(sword32* r, const sword32* a)
     o = (t13 >> 28); t14 += o; t = o << 28; t13 -= t;
     o = (t14 >> 28); t15 += o; t = o << 28; t14 -= t;
     o = (t15 >> 28); t0  += o;
-                   t8  += o; t = o << 28; t15 -= t;
+                     t8  += o; t = o << 28; t15 -= t;
 
     /* Store */
     r[0] = (sword32)t0;
@@ -2168,7 +2165,6 @@ int curve448(byte* r, const byte* n, const byte* a)
     sword32 t1[16];
     int i;
     unsigned int swap;
-    unsigned int b;
 
     fe448_from_bytes(x1, a);
     fe448_1(x2);
@@ -2178,10 +2174,10 @@ int curve448(byte* r, const byte* n, const byte* a)
 
     swap = 0;
     for (i = 447; i >= 0; --i) {
-        b = (n[i >> 3] >> (i & 7)) & 1;
+        unsigned int b = (n[i >> 3] >> (i & 7)) & 1;
         swap ^= b;
-        fe448_cswap(x2, x3, swap);
-        fe448_cswap(z2, z3, swap);
+        fe448_cswap(x2, x3, (int)swap);
+        fe448_cswap(z2, z3, (int)swap);
         swap = b;
 
         /* Montgomery Ladder - double and add */
@@ -2273,7 +2269,7 @@ void fe448_neg(sword32* r, const sword32* a)
 }
 
 /* Raise field element to (p-3) / 4: 2^446 - 2^222 - 1
- * Used for calcualting y-ordinate from x-ordinate for Ed448.
+ * Used for calculating y-ordinate from x-ordinate for Ed448.
  *
  * r  [in]  Field element to hold result.
  * a  [in]  Field element to exponentiate.
