@@ -1,12 +1,12 @@
 /* wc_lms.c
  *
- * Copyright (C) 2006-2024 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -19,13 +19,7 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1335, USA
  */
 
-#ifdef HAVE_CONFIG_H
-    #include <config.h>
-#endif
-
-#include <wolfssl/wolfcrypt/settings.h>
-#include <wolfssl/wolfcrypt/error-crypt.h>
-#include <wolfssl/wolfcrypt/logging.h>
+#include <wolfssl/wolfcrypt/libwolfssl_sources.h>
 
 #if defined(WOLFSSL_HAVE_LMS) && defined(WOLFSSL_WC_LMS)
 #include <wolfssl/wolfcrypt/wc_lms.h>
@@ -42,8 +36,8 @@
  *
  * @param [in] w  Winternitz width.
  */
-#define LMS_U(w)                    \
-    (8 * WC_SHA256_DIGEST_SIZE / (w))
+#define LMS_U(w, hLen)              \
+    (8 * (hLen) / (w))
 /* Calculate u. Appendix B. Works for w of 1, 2, 4, or 8.
  *
  * @param [in] w   Winternitz width.
@@ -63,17 +57,17 @@
  * @param [in] w   Winternitz width.
  * @param [in] wb  Winternitz width length in bits.
  */
-#define LMS_P(w, wb)                \
-    (LMS_U(w) + LMS_V(w, wb))
+#define LMS_P(w, wb, hLen)          \
+    (LMS_U(w, hLen) + LMS_V(w, wb))
 /* Calculate signature length.
  *
  * @param [in] l  Number of levels.
  * @param [in] h  Height of the trees.
  * @param [in] p  Number of n-byte string elements in signature for a tree.
  */
-#define LMS_PARAMS_SIG_LEN(l, h, p)                                     \
-    (4 + (l) * (4 + 4 + 4 + WC_SHA256_DIGEST_SIZE * (1 + (p) + (h))) +  \
-     ((l) - 1) * LMS_PUBKEY_LEN)
+#define LMS_PARAMS_SIG_LEN(l, h, p, hLen)               \
+    (4 + (l) * (4 + 4 + 4 + (hLen) * (1 + (p) + (h))) + \
+     ((l) - 1) * LMS_PUBKEY_LEN(hLen))
 
 #ifndef WOLFSSL_WC_LMS_SMALL
     /* Root levels and leaf cache bits. */
@@ -94,9 +88,10 @@
  * @param [in] t   LMS type.
  * @param [in] t2  LM-OTS type.
  */
-#define LMS_PARAMS(l, h, w, wb, t, t2)                              \
-    { l, h, w, LMS_LS(w, wb), LMS_P(w, wb), t, t2,                  \
-      LMS_PARAMS_SIG_LEN(l, h, LMS_P(w, wb)), LMS_PARAMS_CACHE(h) }
+#define LMS_PARAMS(l, h, w, wb, t, t2, hLen)                \
+    { l, h, w, LMS_LS(w, wb), LMS_P(w, wb, hLen), t, t2,    \
+      LMS_PARAMS_SIG_LEN(l, h, LMS_P(w, wb, hLen), hLen),   \
+      (hLen), LMS_PARAMS_CACHE(h) }
 
 
 /* Initialize the working state for LMS operations.
@@ -138,112 +133,230 @@ static void wc_lmskey_state_free(LmsState* state)
 
 /* Supported LMS parameters. */
 static const wc_LmsParamsMap wc_lms_map[] = {
+#ifndef WOLFSSL_NO_LMS_SHA256_256
 #if LMS_MAX_HEIGHT >= 15
     { WC_LMS_PARM_NONE     , "LMS_NONE"         ,
-      LMS_PARAMS(1, 15, 2, 1, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(1, 15, 2, 1, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H15_W2, "LMS/HSS L1_H15_W2",
-      LMS_PARAMS(1, 15, 2, 1, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(1, 15, 2, 1, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H15_W4, "LMS/HSS L1_H15_W4",
-      LMS_PARAMS(1, 15, 4, 2, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(1, 15, 4, 2, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #if LMS_MAX_LEVELS >= 2
 #if LMS_MAX_HEIGHT >= 10
     { WC_LMS_PARM_L2_H10_W2, "LMS/HSS L2_H10_W2",
-      LMS_PARAMS(2, 10, 2, 1, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(2, 10, 2, 1, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H10_W4, "LMS/HSS L2_H10_W4",
-      LMS_PARAMS(2, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(2, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H10_W8, "LMS/HSS L2_H10_W8",
-      LMS_PARAMS(2, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(2, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #endif
 #if LMS_MAX_LEVELS >= 3
     { WC_LMS_PARM_L3_H5_W2 , "LMS/HSS L3_H5_W2" ,
-      LMS_PARAMS(3,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(3,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L3_H5_W4 , "LMS/HSS L3_H5_W4" ,
-      LMS_PARAMS(3,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(3,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L3_H5_W8 , "LMS/HSS L3_H5_W8" ,
-      LMS_PARAMS(3,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(3,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #if LMS_MAX_HEIGHT >= 10
     { WC_LMS_PARM_L3_H10_W4, "LMS/HSS L3_H10_W4",
-      LMS_PARAMS(3, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(3, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #endif
 #if LMS_MAX_LEVELS >= 4
     { WC_LMS_PARM_L4_H5_W8 , "LMS/HSS L4_H5_W8" ,
-      LMS_PARAMS(4,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(4,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 
     /* For when user sets L, H, W explicitly. */
     { WC_LMS_PARM_L1_H5_W1 , "LMS/HSS_L1_H5_W1" ,
-      LMS_PARAMS(1,  5, 1, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W1) },
+      LMS_PARAMS(1,  5, 1, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W1,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H5_W2 , "LMS/HSS_L1_H5_W2" ,
-      LMS_PARAMS(1,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(1,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H5_W4 , "LMS/HSS_L1_H5_W4" ,
-      LMS_PARAMS(1,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(1,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H5_W8 , "LMS/HSS_L1_H5_W8" ,
-      LMS_PARAMS(1,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(1,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #if LMS_MAX_HEIGHT >= 10
     { WC_LMS_PARM_L1_H10_W2 , "LMS/HSS_L1_H10_W2",
-      LMS_PARAMS(1, 10, 2, 1, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(1, 10, 2, 1, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H10_W4 , "LMS/HSS_L1_H10_W4",
-      LMS_PARAMS(1, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(1, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H10_W8 , "LMS/HSS_L1_H10_W8",
-      LMS_PARAMS(1, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(1, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #if LMS_MAX_HEIGHT >= 15
     { WC_LMS_PARM_L1_H15_W8 , "LMS/HSS L1_H15_W8",
-      LMS_PARAMS(1, 15, 8, 3, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(1, 15, 8, 3, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #if LMS_MAX_HEIGHT >= 20
     { WC_LMS_PARM_L1_H20_W2 , "LMS/HSS_L1_H20_W2",
-      LMS_PARAMS(1, 20, 2, 1, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(1, 20, 2, 1, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H20_W4 , "LMS/HSS_L1_H20_W4",
-      LMS_PARAMS(1, 20, 4, 2, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(1, 20, 4, 2, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L1_H20_W8 , "LMS/HSS_L1_H20_W8",
-      LMS_PARAMS(1, 20, 8, 3, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(1, 20, 8, 3, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #if LMS_MAX_LEVELS >= 2
     { WC_LMS_PARM_L2_H5_W2 , "LMS/HSS_L2_H5_W2" ,
-      LMS_PARAMS(2,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(2,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H5_W4 , "LMS/HSS_L2_H5_W4" ,
-      LMS_PARAMS(2,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(2,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H5_W8 , "LMS/HSS_L2_H5_W8" ,
-      LMS_PARAMS(2,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(2,  5, 8, 3, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #if LMS_MAX_HEIGHT >= 15
     { WC_LMS_PARM_L2_H15_W2 , "LMS/HSS_L2_H15_W2",
-      LMS_PARAMS(2, 15, 2, 1, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(2, 15, 2, 1, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H15_W4 , "LMS/HSS_L2_H15_W4",
-      LMS_PARAMS(2, 15, 4, 2, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(2, 15, 4, 2, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H15_W8 , "LMS/HSS_L2_H15_W8",
-      LMS_PARAMS(2, 15, 8, 3, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(2, 15, 8, 3, LMS_SHA256_M32_H15, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #if LMS_MAX_HEIGHT >= 20
     { WC_LMS_PARM_L2_H20_W2 , "LMS/HSS_L2_H20_W2",
-      LMS_PARAMS(2, 20, 2, 1, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(2, 20, 2, 1, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H20_W4 , "LMS/HSS_L2_H20_W4",
-      LMS_PARAMS(2, 20, 4, 2, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(2, 20, 4, 2, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L2_H20_W8 , "LMS/HSS_L2_H20_W8",
-      LMS_PARAMS(2, 20, 8, 3, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(2, 20, 8, 3, LMS_SHA256_M32_H20, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #endif
 #if LMS_MAX_LEVELS >= 3
 #if LMS_MAX_HEIGHT >= 10
     { WC_LMS_PARM_L3_H10_W8 , "LMS/HSS L3_H10_W8",
-      LMS_PARAMS(3, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(3, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #endif
 #if LMS_MAX_LEVELS >= 4
     { WC_LMS_PARM_L4_H5_W2 , "LMS/HSS L4_H5_W2" ,
-      LMS_PARAMS(4,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2) },
+      LMS_PARAMS(4,  5, 2, 1, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W2,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L4_H5_W4 , "LMS/HSS L4_H5_W4" ,
-      LMS_PARAMS(4,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(4,  5, 4, 2, LMS_SHA256_M32_H5 , LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
 #if LMS_MAX_HEIGHT >= 10
     { WC_LMS_PARM_L4_H10_W4 , "LMS/HSS L4_H10_W4",
-      LMS_PARAMS(4, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4) },
+      LMS_PARAMS(4, 10, 4, 2, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W4,
+                 WC_SHA256_DIGEST_SIZE) },
     { WC_LMS_PARM_L4_H10_W8 , "LMS/HSS L4_H10_W8",
-      LMS_PARAMS(4, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8) },
+      LMS_PARAMS(4, 10, 8, 3, LMS_SHA256_M32_H10, LMOTS_SHA256_N32_W8,
+                 WC_SHA256_DIGEST_SIZE) },
 #endif
 #endif
+#endif /* !WOLFSSL_NO_LMS_SHA256_256 */
+
+#ifdef WOLFSSL_LMS_SHA256_192
+#if LMS_MAX_HEIGHT >= 15
+    { WC_LMS_PARM_SHA256_192_L1_H15_W2, "LMS/HSS_SHA256/192 L1_H15_W2",
+      LMS_PARAMS(1, 15, 2, 2, LMS_SHA256_M24_H15, LMOTS_SHA256_N24_W2,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H15_W4, "LMS/HSS_SHA256/192 L1_H15_W4",
+      LMS_PARAMS(1, 15, 4, 3, LMS_SHA256_M24_H15, LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#endif
+#if LMS_MAX_LEVELS >= 2
+#if LMS_MAX_HEIGHT >= 10
+    { WC_LMS_PARM_SHA256_192_L2_H10_W2, "LMS/HSS SHA256/192 L2_H10_W2",
+      LMS_PARAMS(2, 10, 2, 2, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W2,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L2_H10_W4, "LMS/HSS SHA256/192 L2_H10_W4",
+      LMS_PARAMS(2, 10, 4, 3, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L2_H10_W8, "LMS/HSS SHA256/192 L2_H10_W8",
+      LMS_PARAMS(2, 10, 8, 4, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W8,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#endif
+#endif
+#if LMS_MAX_LEVELS >= 3
+    { WC_LMS_PARM_SHA256_192_L3_H5_W2 , "LMS/HSS_SHA256/192 L3_H5_W2" ,
+      LMS_PARAMS(3,  5, 2, 2, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W2,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L3_H5_W4 , "LMS/HSS_SHA256/192 L3_H5_W4" ,
+      LMS_PARAMS(3,  5, 4, 3, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L3_H5_W8 , "LMS/HSS_SHA256/192 L3_H5_W8" ,
+      LMS_PARAMS(3,  5, 8, 4, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W8,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#if LMS_MAX_HEIGHT >= 10
+    { WC_LMS_PARM_SHA256_192_L3_H10_W4, "LMS/HSS_SHA256/192 L3_H10_W4",
+      LMS_PARAMS(3, 10, 4, 3, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#endif
+#endif
+#if LMS_MAX_LEVELS >= 4
+    { WC_LMS_PARM_SHA256_192_L4_H5_W8 , "LMS/HSS_SHA256/192 L4_H5_W8" ,
+      LMS_PARAMS(4,  5, 8, 4, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W8,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#endif
+
+    { WC_LMS_PARM_SHA256_192_L1_H5_W1 , "LMS/HSS_SHA256/192_L1_H5_W1" ,
+      LMS_PARAMS(1,  5, 1, 2, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W1,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H5_W2 , "LMS/HSS_SHA256/192_L1_H5_W2" ,
+      LMS_PARAMS(1,  5, 2, 2, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W2,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H5_W4 , "LMS/HSS_SHA256/192_L1_H5_W4" ,
+      LMS_PARAMS(1,  5, 4, 3, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H5_W8 , "LMS/HSS_SHA256/192_L1_H5_W8" ,
+      LMS_PARAMS(1,  5, 8, 4, LMS_SHA256_M24_H5 , LMOTS_SHA256_N24_W8,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#if LMS_MAX_HEIGHT >= 10
+    { WC_LMS_PARM_SHA256_192_L1_H10_W2 , "LMS/HSS_SHA256/192_L1_H10_W2",
+      LMS_PARAMS(1, 10, 2, 2, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W2,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H10_W4 , "LMS/HSS_SHA256/192_L1_H10_W4",
+      LMS_PARAMS(1, 10, 4, 3, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H10_W8 , "LMS/HSS_SHA256/192_L1_H10_W8",
+      LMS_PARAMS(1, 10, 8, 4, LMS_SHA256_M24_H10, LMOTS_SHA256_N24_W8,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#endif
+#if LMS_MAX_HEIGHT >= 20
+    { WC_LMS_PARM_SHA256_192_L1_H20_W2 , "LMS/HSS_SHA256/192_L1_H20_W2",
+      LMS_PARAMS(1, 20, 2, 2, LMS_SHA256_M24_H20, LMOTS_SHA256_N24_W2,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H20_W4 , "LMS/HSS_SHA256/192_L1_H20_W4",
+      LMS_PARAMS(1, 20, 4, 3, LMS_SHA256_M24_H20, LMOTS_SHA256_N24_W4,
+                 WC_SHA256_192_DIGEST_SIZE) },
+    { WC_LMS_PARM_SHA256_192_L1_H20_W8 , "LMS/HSS_SHA256/192_L1_H20_W8",
+      LMS_PARAMS(1, 20, 8, 4, LMS_SHA256_M24_H20, LMOTS_SHA256_N24_W8,
+                 WC_SHA256_192_DIGEST_SIZE) },
+#endif
+#endif /* WOLFSSL_LMS_SHA256_192 */
 };
 /* Number of parameter sets supported. */
 #define WC_LMS_MAP_LEN      ((int)(sizeof(wc_lms_map) / sizeof(*wc_lms_map)))
@@ -473,11 +586,14 @@ void wc_LmsKey_Free(LmsKey* key)
     #ifndef WOLFSSL_LMS_VERIFY_ONLY
         if (key->priv_data != NULL) {
             const LmsParams* params = key->params;
-
-            ForceZero(key->priv_data, LMS_PRIV_DATA_LEN(params->levels,
+            int priv_data_len = LMS_PRIV_DATA_LEN(params->levels,
                 params->height, params->p, params->rootLevels,
-                params->cacheBits));
+                params->cacheBits, params->hash_len);
 
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+            priv_data_len += HSS_PRIVATE_KEY_LEN(key->params->hash_len);
+#endif
+            ForceZero(key->priv_data, priv_data_len);
             XFREE(key->priv_data, key->heap, DYNAMIC_TYPE_LMS);
         }
     #endif
@@ -604,6 +720,7 @@ int wc_LmsKey_SetContext(LmsKey* key, void* context)
 int wc_LmsKey_MakeKey(LmsKey* key, WC_RNG* rng)
 {
     int ret = 0;
+    int priv_data_len = 0;
 
     /* Validate parameters. */
     if ((key == NULL) || (rng == NULL)) {
@@ -625,17 +742,26 @@ int wc_LmsKey_MakeKey(LmsKey* key, WC_RNG* rng)
         ret = BAD_FUNC_ARG;
     }
 
-    if ((ret == 0) && (key->priv_data == NULL)) {
+    if (ret == 0) {
         const LmsParams* params = key->params;
+        priv_data_len = LMS_PRIV_DATA_LEN(params->levels, params->height,
+            params->p, params->rootLevels, params->cacheBits, params->hash_len);
 
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        priv_data_len += HSS_PRIVATE_KEY_LEN(key->params->hash_len);
+#endif
+    }
+    if ((ret == 0) && (key->priv_data == NULL)) {
         /* Allocate memory for the private key data. */
-        key->priv_data = XMALLOC(LMS_PRIV_DATA_LEN(params->levels,
-            params->height, params->p, params->rootLevels, params->cacheBits),
-            key->heap, DYNAMIC_TYPE_LMS);
+        key->priv_data = (byte *)XMALLOC(priv_data_len, key->heap,
+            DYNAMIC_TYPE_LMS);
         /* Check pointer is valid. */
         if (key->priv_data == NULL) {
             ret = MEMORY_E;
         }
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        XMEMSET(key->priv_data, 0, priv_data_len);
+#endif
     }
     if (ret == 0) {
     #ifdef WOLFSSL_SMALL_STACK
@@ -646,7 +772,8 @@ int wc_LmsKey_MakeKey(LmsKey* key, WC_RNG* rng)
 
     #ifdef WOLFSSL_SMALL_STACK
         /* Allocate memory for working state. */
-        state = XMALLOC(sizeof(LmsState), NULL, DYNAMIC_TYPE_TMP_BUFFER);
+        state = (LmsState*)XMALLOC(sizeof(LmsState), NULL,
+            DYNAMIC_TYPE_TMP_BUFFER);
         if (state == NULL) {
             ret = MEMORY_E;
         }
@@ -668,9 +795,18 @@ int wc_LmsKey_MakeKey(LmsKey* key, WC_RNG* rng)
         }
     }
     if (ret == 0) {
+        int rv;
         /* Write private key to storage. */
-        int rv = key->write_private_key(key->priv_raw, HSS_PRIVATE_KEY_LEN,
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        XMEMCPY(key->priv_data + priv_data_len -
+            HSS_PRIVATE_KEY_LEN(key->params->hash_len), key->priv_raw,
+            HSS_PRIVATE_KEY_LEN(key->params->hash_len));
+        rv = key->write_private_key(key->priv_data, priv_data_len,
             key->context);
+#else
+        rv = key->write_private_key(key->priv_raw,
+            HSS_PRIVATE_KEY_LEN(key->params->hash_len), key->context);
+#endif
         if (rv != WC_LMS_RC_SAVED_TO_NV_MEMORY) {
             ret = IO_FAILED_E;
         }
@@ -703,6 +839,7 @@ int wc_LmsKey_MakeKey(LmsKey* key, WC_RNG* rng)
 int wc_LmsKey_Reload(LmsKey* key)
 {
     int ret = 0;
+    int priv_data_len = 0;
 
     /* Validate parameter. */
     if (key == NULL) {
@@ -724,25 +861,46 @@ int wc_LmsKey_Reload(LmsKey* key)
         ret = BAD_FUNC_ARG;
     }
 
-    if ((ret == 0) && (key->priv_data == NULL)) {
+    if (ret == 0) {
         const LmsParams* params = key->params;
+        priv_data_len = LMS_PRIV_DATA_LEN(params->levels, params->height,
+            params->p, params->rootLevels, params->cacheBits, params->hash_len);
 
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        priv_data_len += HSS_PRIVATE_KEY_LEN(params->hash_len);
+#endif
+    }
+    if ((ret == 0) && (key->priv_data == NULL)) {
         /* Allocate memory for the private key data. */
-        key->priv_data = XMALLOC(LMS_PRIV_DATA_LEN(params->levels,
-            params->height, params->p, params->rootLevels, params->cacheBits),
-            key->heap, DYNAMIC_TYPE_LMS);
+        key->priv_data = (byte *)XMALLOC(priv_data_len, key->heap,
+            DYNAMIC_TYPE_LMS);
         /* Check pointer is valid. */
         if (key->priv_data == NULL) {
             ret = MEMORY_E;
         }
     }
     if (ret == 0) {
+        int rv;
+
         /* Load private key. */
-        int rv = key->read_private_key(key->priv_raw, HSS_PRIVATE_KEY_LEN,
-            key->context);
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        const LmsParams* params = key->params;
+
+        rv = key->read_private_key(key->priv_data, priv_data_len, key->context);
+#else
+        rv = key->read_private_key(key->priv_raw,
+            HSS_PRIVATE_KEY_LEN(key->params->hash_len), key->context);
+#endif
         if (rv != WC_LMS_RC_READ_TO_MEMORY) {
             ret = IO_FAILED_E;
         }
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        if (ret == 0) {
+            XMEMCPY(key->priv_raw, key->priv_data + priv_data_len -
+                HSS_PRIVATE_KEY_LEN(params->hash_len),
+                HSS_PRIVATE_KEY_LEN(params->hash_len));
+        }
+#endif
     }
 
     /* Double check the key actually has signatures left. */
@@ -761,7 +919,8 @@ int wc_LmsKey_Reload(LmsKey* key)
 
     #ifdef WOLFSSL_SMALL_STACK
         /* Allocate memory for working state. */
-        state = XMALLOC(sizeof(LmsState), NULL, DYNAMIC_TYPE_TMP_BUFFER);
+        state = (LmsState*)XMALLOC(sizeof(LmsState), NULL,
+            DYNAMIC_TYPE_TMP_BUFFER);
         if (state == NULL) {
             ret = MEMORY_E;
         }
@@ -808,7 +967,7 @@ int wc_LmsKey_GetPrivLen(const LmsKey* key, word32* len)
 
     if (ret == 0) {
         /* Return private key length from parameter set. */
-        *len = HSS_PRIVATE_KEY_LEN;
+        *len = HSS_PRIVATE_KEY_LEN(key->params->hash_len);
     }
 
     return ret;
@@ -859,7 +1018,8 @@ int wc_LmsKey_Sign(LmsKey* key, byte* sig, word32* sigSz, const byte* msg,
 
     #ifdef WOLFSSL_SMALL_STACK
         /* Allocate memory for working state. */
-        state = XMALLOC(sizeof(LmsState), NULL, DYNAMIC_TYPE_TMP_BUFFER);
+        state = (LmsState*)XMALLOC(sizeof(LmsState), NULL,
+            DYNAMIC_TYPE_TMP_BUFFER);
         if (state == NULL) {
             ret = MEMORY_E;
         }
@@ -884,9 +1044,24 @@ int wc_LmsKey_Sign(LmsKey* key, byte* sig, word32* sigSz, const byte* msg,
         *sigSz = (word32)key->params->sig_len;
     }
     if (ret == 0) {
+        int rv;
+
         /* Write private key to storage. */
-        int rv = key->write_private_key(key->priv_raw, HSS_PRIVATE_KEY_LEN,
+#ifdef WOLFSSL_WC_LMS_SERIALIZE_STATE
+        const LmsParams* params = key->params;
+        int priv_data_len = LMS_PRIV_DATA_LEN(params->levels, params->height,
+            params->p, params->rootLevels, params->cacheBits,
+            params->hash_len) + HSS_PRIVATE_KEY_LEN(key->params->hash_len);
+
+        XMEMCPY(key->priv_data + priv_data_len -
+            HSS_PRIVATE_KEY_LEN(params->hash_len), key->priv_raw,
+            HSS_PRIVATE_KEY_LEN(params->hash_len));
+        rv = key->write_private_key(key->priv_data, priv_data_len,
             key->context);
+#else
+        rv = key->write_private_key(key->priv_raw,
+            HSS_PRIVATE_KEY_LEN(key->params->hash_len), key->context);
+#endif
         if (rv != WC_LMS_RC_SAVED_TO_NV_MEMORY) {
             ret = IO_FAILED_E;
         }
@@ -933,7 +1108,7 @@ int wc_LmsKey_GetPubLen(const LmsKey* key, word32* len)
     }
 
     if (ret == 0) {
-        *len = HSS_PUBLIC_KEY_LEN;
+        *len = HSS_PUBLIC_KEY_LEN(key->params->hash_len);
     }
 
     return ret;
@@ -996,14 +1171,15 @@ int wc_LmsKey_ExportPubRaw(const LmsKey* key, byte* out, word32* outLen)
         ret = BAD_FUNC_ARG;
     }
     /* Check size of out is sufficient. */
-    if ((ret == 0) && (*outLen < HSS_PUBLIC_KEY_LEN)) {
+    if ((ret == 0) &&
+            (*outLen < (word32)HSS_PUBLIC_KEY_LEN(key->params->hash_len))) {
         ret = BUFFER_E;
     }
 
     if (ret == 0) {
         /* Return encoded public key. */
-        XMEMCPY(out, key->pub, HSS_PUBLIC_KEY_LEN);
-        *outLen = HSS_PUBLIC_KEY_LEN;
+        XMEMCPY(out, key->pub, HSS_PUBLIC_KEY_LEN(key->params->hash_len));
+        *outLen = HSS_PUBLIC_KEY_LEN(key->params->hash_len);
     }
 
     return ret;
@@ -1032,7 +1208,8 @@ int wc_LmsKey_ImportPubRaw(LmsKey* key, const byte* in, word32 inLen)
     if ((key == NULL) || (in == NULL)) {
         ret = BAD_FUNC_ARG;
     }
-    if ((ret == 0) && (inLen != HSS_PUBLIC_KEY_LEN)) {
+    if ((ret == 0) &&
+            (inLen != (word32)HSS_PUBLIC_KEY_LEN(key->params->hash_len))) {
         /* Something inconsistent. Parameters weren't set, or input
          * pub key is wrong.*/
         return BUFFER_E;
@@ -1041,7 +1218,8 @@ int wc_LmsKey_ImportPubRaw(LmsKey* key, const byte* in, word32 inLen)
     if (ret == 0) {
         XMEMCPY(key->pub, in, inLen);
 
-        key->state = WC_LMS_STATE_VERIFYONLY;
+        if (key->state != WC_LMS_STATE_OK)
+            key->state = WC_LMS_STATE_VERIFYONLY;
     }
 
     return ret;
@@ -1118,7 +1296,8 @@ int wc_LmsKey_Verify(LmsKey* key, const byte* sig, word32 sigSz,
 
     #ifdef WOLFSSL_SMALL_STACK
         /* Allocate memory for working state. */
-        state = XMALLOC(sizeof(LmsState), NULL, DYNAMIC_TYPE_TMP_BUFFER);
+        state = (LmsState*)XMALLOC(sizeof(LmsState), NULL,
+            DYNAMIC_TYPE_TMP_BUFFER);
         if (state == NULL) {
             ret = MEMORY_E;
         }
@@ -1142,4 +1321,60 @@ int wc_LmsKey_Verify(LmsKey* key, const byte* sig, word32 sigSz,
     return ret;
 }
 
+#ifndef WOLFSSL_LMS_VERIFY_ONLY
+
+/* Get the Key ID from the LMS key.
+ *
+ * PRIV = Q | PARAMS | SEED | I
+ * where I is the Key ID.
+ *
+ * @param [in]  key    LMS key.
+ * @param [out] kid    Key ID data.
+ * @param [out] kidSz  Size of key ID.
+ * @return  0 on success.
+ * @return  BAD_FUNC_ARG when a key, kid or kidSz is NULL.
+ */
+int wc_LmsKey_GetKid(LmsKey * key, const byte ** kid, word32* kidSz)
+{
+    word32 offset;
+
+    if ((key == NULL) || (kid == NULL) || (kidSz == NULL)) {
+        return BAD_FUNC_ARG;
+    }
+
+    /* SEED length is hash length. */
+    offset = HSS_Q_LEN + HSS_PRIV_KEY_PARAM_SET_LEN + key->params->hash_len;
+    *kid = key->priv_raw + offset;
+    *kidSz = HSS_PRIVATE_KEY_LEN(key->params->hash_len) - offset;
+
+    return 0;
+}
+
+
+/* Get the Key ID from the raw private key data.
+ *
+ * PRIV = Q | PARAMS | SEED | I
+ * where I is the Key ID.
+ *
+ * @param [in] priv    Private key data.
+ * @param [in] privSz  Size of private key data.
+ * @param  Pointer to 16 byte Key ID in the private key.
+ * @return  NULL on failure.
+ */
+const byte * wc_LmsKey_GetKidFromPrivRaw(const byte * priv, word32 privSz)
+{
+    word32 seedSz = privSz - HSS_Q_LEN - HSS_PRIV_KEY_PARAM_SET_LEN - LMS_I_LEN;
+
+    if (priv == NULL) {
+        return NULL;
+    }
+    if ((seedSz != WC_SHA256_192_DIGEST_SIZE) &&
+            (seedSz != WC_SHA256_DIGEST_SIZE)) {
+        return NULL;
+    }
+    return priv + privSz - LMS_I_LEN;
+}
+
+#endif
+
 #endif /* WOLFSSL_HAVE_LMS && WOLFSSL_WC_LMS */
