@@ -1,12 +1,12 @@
 /* renesas-tsip-crypt.h
  *
- * Copyright (C) 2006-2023 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -35,9 +35,17 @@
     #include "r_tsip_rx_if.h"
 #endif
 
-
+#include <wolfssl/wolfcrypt/settings.h>
+#include <wolfssl/wolfcrypt/types.h>
 #include <wolfssl/wolfcrypt/logging.h>
+#include <wolfssl/wolfcrypt/hash.h>
+#ifdef WOLFSSL_RENESAS_TSIP_TLS
 #include <wolfssl/ssl.h>
+#endif
+#ifdef WOLF_CRYPTO_CB
+#include <wolfssl/wolfcrypt/cryptocb.h>
+#endif
+
 
 #ifdef __cplusplus
 extern "C" {
@@ -51,7 +59,7 @@ extern "C" {
 typedef enum {
     WOLFSSL_TSIP_NOERROR = 0,
     WOLFSSL_TSIP_ILLEGAL_CIPHERSUITE = 0xffffffff,
-}wolfssl_tsip_error_number;
+} wolfssl_tsip_error_number;
 
 typedef enum {
     tsip_Key_SESSION = 1,
@@ -84,19 +92,24 @@ enum {
     TSIP_TLS_VERIFY_DATA_WD_SZ = 8,
     TSIP_TLS_MAX_SIGDATA_SZ    = 130,
     TSIP_TEMP_WORK_SIZE        = 128,
+
+    TSIP_MAX_ECC_BYTES         = 48,
 };
 
 typedef enum {
-    TSIP_KEY_TYPE_RSA2048      = 0,
-    TSIP_KEY_TYPE_RSA4096      = 1,
-    TSIP_KEY_TYPE_ECDSAP256    = 2,
     #ifdef WOLFSSL_RENESAS_TSIP_CRYPTONLY
-    TSIP_KEY_TYPE_RSA1024      = 3,
+    TSIP_KEY_TYPE_RSA1024      = 1, /* RSA 1024 */
     #endif
-
+    TSIP_KEY_TYPE_RSA2048      = 2, /* RSA 2048 */
+    TSIP_KEY_TYPE_RSA3072      = 3, /* RSA 3072 */
+    TSIP_KEY_TYPE_RSA4096      = 4, /* RSA 4096 */
+    TSIP_KEY_TYPE_ECDSAP256    = 5, /* ECC P256 */
+    TSIP_KEY_TYPE_ECDSAP384    = 6, /* ECC P384 */
 } wolfssl_TSIP_KEY_TYPE;
 
+
 struct WOLFSSL;
+struct ecc_key;
 struct KeyShareEntry;
 
 /*  MsgBag stands for message bag and acts as a buffer for holding plain text
@@ -121,30 +134,38 @@ typedef struct MsgBag
 } MsgBag;
 
 #ifdef WOLFSSL_RENESAS_TSIP_CRYPTONLY
-
- typedef void* renesas_tsip_key;
-
- /* flags Crypt Only */
- struct tsip_keyflgs_cryt {
-    uint8_t aes256_key_set:1;
-    uint8_t aes128_key_set:1;
-    uint8_t rsapri2048_key_set:1;
-    uint8_t rsapub2048_key_set:1;
-    uint8_t rsapri1024_key_set:1;
-    uint8_t rsapub1024_key_set:1;
-    uint8_t message_type:1;/*message 0, hashed 1*/
- };
+/* flags Crypt Only */
+struct tsip_keyflgs_crypt {
+    uint32_t aes256_key_set:1;
+    uint32_t aes128_key_set:1;
+    uint32_t rsapri2048_key_set:1;
+    uint32_t rsapub2048_key_set:1;
+    uint32_t rsapri1024_key_set:1;
+    uint32_t rsapub1024_key_set:1;
+    uint32_t eccpri_key_set:1;
+    uint32_t eccpub_key_set:1;
+    uint32_t message_type:1; /*message 0, hashed 1*/
+};
 #endif
+
 /*
  * TsipUserCtx holds mainly keys used for TLS handshake in TSIP specific format.
  */
 typedef struct TsipUserCtx {
     /* unique number for each session */
     int devId;
+
+    /* client key pair wrapped by provisioning key */
+    byte*                                              wrappedPrivateKey;
+    byte*                                              wrappedPublicKey;
+
+    int                                                wrappedKeyType;
+
 #ifdef WOLFSSL_RENESAS_TSIP_TLS
     /* 0:working as a TLS client, 1: as a server */
     byte                    side;
-
+    /* ENCRYPT_SIDE_ONLY:1 DECRYPT_SIDE_ONLY:2 ENCRYPT AND DECRYPT:3 */
+    byte                    key_side;
     /* public key index for verification of RootCA cert */
     uint32_t                user_key_id;
 
@@ -159,24 +180,17 @@ typedef struct TsipUserCtx {
 
     /* handle is used as work area for Tls13 handshake */
     tsip_tls13_handle_t                                handle13;
-#endif /* WOLFSSL_RENESAS_TSIP_TLS */
-    /* client key pair wrapped by provisioning key */
-    byte*                                              wrappedPrivateKey;
-    byte*                                              wrappedPublicKey;
 
-    int                                                wrappedKeyType;
-#ifdef WOLFSSL_RENESAS_TSIP_TLS
-    #if !defined(NO_RSA)
+#if !defined(NO_RSA)
     /* RSA-2048bit private and public key-index for client authentication */
     tsip_rsa2048_private_key_index_t                   Rsa2048PrivateKeyIdx;
     tsip_rsa2048_public_key_index_t                    Rsa2048PublicKeyIdx;
-    #endif /* !NO_RSA */
-
-    #if defined(HAVE_ECC)
-    /* ECC P256 private and public key-index for client authentication */
-    tsip_ecc_private_key_index_t                       EcdsaP256PrivateKeyIdx;
-    tsip_ecc_public_key_index_t                        EcdsaP256PublicKeyIdx;
-    #endif /* HAVE_ECC */
+#endif /* !NO_RSA */
+#if defined(HAVE_ECC)
+    /* ECC private and public key-index for client authentication */
+    tsip_ecc_private_key_index_t                       EcdsaPrivateKeyIdx;
+    tsip_ecc_public_key_index_t                        EcdsaPublicKeyIdx;
+#endif /* HAVE_ECC */
 
     /* ECDHE private key index for Tls13 handshake */
     tsip_tls_p256_ecc_key_index_t                      EcdhPrivKey13Idx;
@@ -226,7 +240,6 @@ typedef struct TsipUserCtx {
     /* signature data area for TLS1.3 CertificateVerify message  */
     byte                             sigDataCertVerify[TSIP_TLS_MAX_SIGDATA_SZ];
 
-
 #if (WOLFSSL_RENESAS_TSIP_VER >=109)
     /* out from R_SCE_TLS_ServerKeyExchangeVerify */
     uint32_t encrypted_ephemeral_ecdh_public_key[ENCRYPTED_ECDHE_PUBKEY_SZ];
@@ -248,47 +261,19 @@ typedef struct TsipUserCtx {
     uint32_t    tsip_masterSecret[TSIP_TLS_MASTERSECRET_SIZE/4];
     uint8_t     tsip_clientRandom[TSIP_TLS_CLIENTRANDOM_SZ];
     uint8_t     tsip_serverRandom[TSIP_TLS_SERVERRANDOM_SZ];
-#endif /* WOLFSSL_RENESAS_TSIP_TLS */
-/* for tsip crypt only mode */
-#ifdef WOLFSSL_RENESAS_TSIP_CRYPTONLY
-
-    renesas_tsip_key    rsa1024pri_keyIdx;
-    renesas_tsip_key    rsa1024pub_keyIdx;
-    renesas_tsip_key    rsa2048pri_keyIdx;
-    renesas_tsip_key    rsa2048pub_keyIdx;
-
-    /* sign/verify hash type :
-     * md5, sha1 or sha256
-     */
-    int sing_hash_type;
-
-    /* flags shows status if tsip keys are installed */
-    union {
-        uint8_t chr;
-        struct tsip_keyflgs_cryt bits;
-    } keyflgs_crypt;
-
-#endif
-    /* installed key handling */
-    tsip_aes_key_index_t user_aes256_key_index;
-    uint8_t user_aes256_key_set:1;
-    tsip_aes_key_index_t user_aes128_key_index;
-    uint8_t user_aes128_key_set:1;
 
     /* TSIP defined cipher suite number */
     uint32_t    tsip_cipher;
 
     /* flags */
-#ifdef WOLFSSL_RENESAS_TSIP_TLS
-    #if !defined(NO_RSA)
+#if !defined(NO_RSA)
     uint8_t ClientRsa2048PrivKey_set:1;
     uint8_t ClientRsa2048PubKey_set:1;
-    #endif
-
-    #if defined(HAVE_ECC)
-    uint8_t ClientEccP256PrivKey_set:1;
-    uint8_t ClientEccP256PubKey_set:1;
-    #endif
+#endif
+#if defined(HAVE_ECC)
+    uint8_t ClientEccPrivKey_set:1;
+    uint8_t ClientEccPubKey_set:1;
+#endif
 
     uint8_t HmacInitialized:1;
     uint8_t RootCAverified:1;
@@ -308,6 +293,43 @@ typedef struct TsipUserCtx {
     uint8_t session_key_set:1;
 #endif /* WOLFSSL_RENESAS_TSIP_TLS */
 
+    /* installed key handling */
+    tsip_aes_key_index_t user_aes256_key_index;
+    uint8_t user_aes256_key_set:1;
+    tsip_aes_key_index_t user_aes128_key_index;
+    uint8_t user_aes128_key_set:1;
+
+/* for tsip crypt only mode */
+#ifdef WOLFSSL_RENESAS_TSIP_CRYPTONLY
+#ifndef NO_RSA
+    #if defined(TSIP_RSAES_1024) && TSIP_RSAES_1024 == 1
+    tsip_rsa1024_private_key_index_t* rsa1024pri_keyIdx;
+    tsip_rsa1024_public_key_index_t*  rsa1024pub_keyIdx;
+    #endif
+    #if defined(TSIP_RSAES_2048) && TSIP_RSAES_2048 == 1
+    tsip_rsa2048_private_key_index_t* rsa2048pri_keyIdx;
+    tsip_rsa2048_public_key_index_t*  rsa2048pub_keyIdx;
+    #endif
+#endif
+#ifdef HAVE_ECC
+    #ifdef HAVE_ECC_SIGN
+    tsip_ecc_private_key_index_t      eccpri_keyIdx;
+    #endif
+    tsip_ecc_public_key_index_t       eccpub_keyIdx;
+#endif
+
+    /* sign/verify hash type :
+     * md5, sha1 or sha256
+     */
+    int sign_hash_type;
+
+    /* flags shows status if tsip keys are installed */
+    union {
+        uint32_t chr;
+        struct tsip_keyflgs_crypt bits;
+    } keyflgs_crypt;
+#endif /* WOLFSSL_RENESAS_TSIP_CRYPTONLY */
+
 } TsipUserCtx;
 
 typedef TsipUserCtx RenesasUserCtx;
@@ -319,37 +341,33 @@ typedef struct
 } TsipPKCbInfo;
 
 
+typedef struct
+{
 #if (WOLFSSL_RENESAS_TSIP_VER >=109)
-
-typedef struct
-{
-    uint8_t *  encrypted_provisioning_key;
-    uint8_t *  iv;
-    uint8_t *  encrypted_user_tls_key;
-    uint32_t   encrypted_user_tls_key_type;
-    uint8_t *  encrypted_user_private_key;
-    uint32_t   encrypted_user_private_key_type;
-    uint8_t *  encrypted_user_public_key;
-    uint32_t   encrypted_user_public_key_type;
+    uint8_t * encrypted_provisioning_key;
+    uint8_t * iv;
+    uint8_t * encrypted_user_tls_key;
+    uint32_t  encrypted_user_tls_key_type;
+    uint8_t * encrypted_user_private_key;
+    uint32_t  encrypted_user_private_key_type;
+    uint8_t * encrypted_user_public_key;
+    uint32_t  encrypted_user_public_key_type;
     tsip_ecc_private_key_index_t client_private_key_index;
-    tsip_tls_ca_certification_public_key_index_t  user_rsa2048_tls_pubindex;
-} tsip_key_data;
-
 #else
-typedef struct
-{
-    uint8_t*  encrypted_session_key;
-    uint8_t*  iv;
-    uint8_t*  encrypted_user_tls_key;
+    uint8_t* encrypted_session_key;
+    uint8_t* iv;
+    uint8_t* encrypted_user_tls_key;
+#endif
     tsip_tls_ca_certification_public_key_index_t  user_rsa2048_tls_pubindex;
 } tsip_key_data;
 
-#endif
 
 struct Aes;
 struct WOLFSSL;
 struct WOLFSSL_CTX;
 struct wc_CryptoInfo;
+
+
 /*----------------------------------------------------*/
 /*   APIs                                             */
 /*----------------------------------------------------*/
@@ -362,6 +380,10 @@ WOLFSSL_API int  tsip_set_callback_ctx(struct WOLFSSL* ssl, void* user_ctx);
 
 WOLFSSL_API int  tsip_set_clientPrivateKeyEnc(const byte* key, int keyType);
 
+
+WOLFSSL_LOCAL int tsip_ImportPublicKey(TsipUserCtx* tuc, int keyType);
+WOLFSSL_LOCAL int tsip_ImportPrivateKey(TsipUserCtx* tuc, int keyType);
+
 #if defined(WOLF_PRIVATE_KEY_ID)
 
 #if defined(WOLFSSL_RENESAS_TSIP_TLS)
@@ -372,7 +394,7 @@ WOLFSSL_API int tsip_use_PrivateKey_buffer_TLS(struct WOLFSSL* ssl,
 #endif
 
 #if defined(WOLFSSL_RENESAS_TSIP_CRYPTONLY)
-WOLFSSL_API int tsip_use_PubicKey_buffer_crypt(TsipUserCtx *uc,
+WOLFSSL_API int tsip_use_PublicKey_buffer_crypt(TsipUserCtx *uc,
                                 const char* keyBuf, int keyBufLen, int keyType);
 WOLFSSL_API int tsip_use_PrivateKey_buffer_crypt(TsipUserCtx *uc,
                                 const char* keyBuf, int keyBufLen, int keyType);
@@ -401,22 +423,32 @@ WOLFSSL_API void tsip_inform_user_keys(
 /*----------------------------------------------------*/
 /*   internal use functions                           */
 /*----------------------------------------------------*/
-WOLFSSL_LOCAL int tsip_SignRsaPkcs(wc_CryptoInfo* info, TsipUserCtx* tuc);
-
+#ifdef HAVE_PK_CALLBACKS
 WOLFSSL_LOCAL int tsip_VerifyRsaPkcsCb(
-                        WOLFSSL* ssl,
+                        struct WOLFSSL* ssl,
                         unsigned char* sig, unsigned int sigSz,
                         unsigned char** out,
                         const unsigned char* keyDer, unsigned int keySz,
                         void* ctx);
 
-WOLFSSL_LOCAL int tsip_SignEcdsa(wc_CryptoInfo* info, TsipUserCtx* tuc);
-
+#endif
 
 #ifdef WOLF_CRYPTO_CB
-
 struct wc_CryptoInfo;
 
+WOLFSSL_LOCAL int tsip_SignRsaPkcs(struct wc_CryptoInfo* info,
+    TsipUserCtx* tuc);
+
+WOLFSSL_API int wc_tsip_RsaVerifyPkcs(struct wc_CryptoInfo* info,
+    TsipUserCtx* tuc);
+
+WOLFSSL_API int wc_tsip_RsaFunction(wc_CryptoInfo* info, TsipUserCtx* tuc);
+
+WOLFSSL_LOCAL int tsip_SignEcdsa(struct wc_CryptoInfo* info, TsipUserCtx* tuc);
+
+WOLFSSL_LOCAL int tsip_VerifyEcdsa(struct wc_CryptoInfo* info,
+    TsipUserCtx* tuc);
+
 WOLFSSL_LOCAL int tsip_TlsCleanup(struct WOLFSSL* ssl);
 
 WOLFSSL_LOCAL int tsip_StoreMessage(struct WOLFSSL* ssl, const byte* data,
@@ -483,9 +515,9 @@ WOLFSSL_LOCAL int tsip_Tls13SendCertVerify(struct WOLFSSL*ssl);
 
 
 #if (WOLFSSL_RENESAS_TSIP_VER >=109)
-WOLFSSL_LOCAL int wc_tsip_AesCipher(int devIdArg, struct wc_CryptoInfo* info,
+WOLFSSL_API int wc_tsip_AesCipher(int devIdArg, struct wc_CryptoInfo* info,
                                                                     void* ctx);
-WOLFSSL_LOCAL int wc_tsip_generateMasterSecretEx(
+WOLFSSL_API int wc_tsip_generateMasterSecretEx(
         byte        cipherSuiteFirst,
         byte        cipherSuite,
         const byte* pr,                 /* pre-master    */
@@ -495,7 +527,7 @@ WOLFSSL_LOCAL int wc_tsip_generateMasterSecretEx(
 
 #else
 
-WOLFSSL_LOCAL int wc_tsip_generateMasterSecret(
+WOLFSSL_API int wc_tsip_generateMasterSecret(
         const byte *pre,
         const byte *cr,
         const byte *sr,
@@ -504,24 +536,24 @@ WOLFSSL_LOCAL int wc_tsip_generateMasterSecret(
 #endif /* WOLFSSL_RENESAS_TSIP_VER */
 
 
-WOLFSSL_LOCAL int wc_tsip_storeKeyCtx(
-        WOLFSSL *ssl,
+WOLFSSL_API int wc_tsip_storeKeyCtx(
+        struct WOLFSSL *ssl,
         TsipUserCtx *userCtx);
 
-WOLFSSL_LOCAL int wc_tsip_generateEncryptPreMasterSecret(
-        WOLFSSL*  ssl,
+WOLFSSL_API int wc_tsip_generateEncryptPreMasterSecret(
+        struct WOLFSSL*  ssl,
         byte*       out,
         word32*     outSz);
 
-WOLFSSL_LOCAL int wc_tsip_EccSharedSecret(
-        WOLFSSL* ssl,
+WOLFSSL_API int wc_tsip_EccSharedSecret(
+        struct WOLFSSL* ssl,
         struct ecc_key* otherKey,
         unsigned char* pubKeyDer, unsigned int* pubKeySz,
         unsigned char* out, unsigned int* outlen,
         int side, void* ctx);
 
-WOLFSSL_LOCAL int wc_tsip_RsaVerify(
-        WOLFSSL* ssl,
+WOLFSSL_API int wc_tsip_RsaVerify(
+        struct WOLFSSL* ssl,
         byte* sig,
         word32 sigSz,
         byte** out,
@@ -529,79 +561,97 @@ WOLFSSL_LOCAL int wc_tsip_RsaVerify(
         word32 keySz,
         void* ctx);
 
-WOLFSSL_LOCAL int wc_tsip_EccVerify(
-        WOLFSSL*  ssl,
+WOLFSSL_API int wc_tsip_EccVerify(
+        struct WOLFSSL*  ssl,
         const byte* sig,    word32  sigSz,
         const byte* hash,   word32  hashSz,
         const byte* key,    word32  keySz,
         int*  result, void*   ctx);
 
-WOLFSSL_LOCAL int wc_tsip_generateVerifyData(
+WOLFSSL_API int wc_tsip_generateVerifyData(
         const uint8_t*  masterSecret,
         const uint8_t*  side,
         const uint8_t*  handshake_hash,
         uint8_t*        hashes);
+
 #ifndef NO_AES
-WOLFSSL_LOCAL int wc_tsip_AesCbcEncrypt(
-        Aes* aes,
+#ifdef HAVE_AES_CBC
+WOLFSSL_API int wc_tsip_AesCbcEncrypt(
+        struct Aes* aes,
         byte*       out,
         const byte* in,
         word32      sz);
 
-WOLFSSL_LOCAL int wc_tsip_AesCbcDecrypt(
-        Aes* aes,
+WOLFSSL_API int wc_tsip_AesCbcDecrypt(
+        struct Aes* aes,
         byte*       out,
         const byte* in,
         word32      sz);
+#endif /* HAVE_AES_CBC */
 
-WOLFSSL_LOCAL int wc_tsip_AesGcmEncrypt(
-        Aes* aes, byte* out,
+#ifdef WOLFSSL_AES_COUNTER
+WOLFSSL_API int wc_tsip_AesCtr(
+        struct Aes*,
+        byte*       out,
+        const byte* in,
+        word32      sz);
+#endif /* WOLFSSL_AES_COUNTER */
+
+#ifdef HAVE_AESGCM
+WOLFSSL_API int wc_tsip_AesGcmEncrypt(
+        struct Aes* aes, byte* out,
         const byte* in, word32 sz,
               byte* iv, word32 ivSz,
               byte* authTag, word32 authTagSz,
         const byte* authIn, word32 authInSz,
         void* ctx);
 
-WOLFSSL_LOCAL int wc_tsip_AesGcmDecrypt(
-        Aes* aes, byte* out,
+WOLFSSL_API int wc_tsip_AesGcmDecrypt(
+        struct Aes* aes, byte* out,
         const byte* in, word32 sz,
         const byte* iv, word32 ivSz,
         const byte* authTag, word32 authTagSz,
         const byte* authIn, word32 authInSz,
         void* ctx);
-#endif /* NO_AES */
-WOLFSSL_LOCAL int wc_tsip_ShaXHmacVerify(
-        const WOLFSSL *ssl,
+#endif /* HAVE_AESGCM */
+#endif /* !NO_AES */
+
+WOLFSSL_API int wc_tsip_ShaXHmacVerify(
+        const struct WOLFSSL *ssl,
         const byte* message,
         word32      messageSz,
         word32      macSz,
         word32      content);
 
-WOLFSSL_LOCAL int wc_tsip_Sha1HmacGenerate(
-        const WOLFSSL *ssl,
+WOLFSSL_API int wc_tsip_Sha1HmacGenerate(
+        const struct WOLFSSL *ssl,
         const byte* myInner,
         word32      innerSz,
         const byte* in,
         word32      sz,
         byte*       digest);
 
-WOLFSSL_LOCAL int wc_tsip_Sha256HmacGenerate(
-        const WOLFSSL *ssl,
+WOLFSSL_API int wc_tsip_Sha256HmacGenerate(
+        const struct WOLFSSL *ssl,
         const byte* myInner,
         word32      innerSz,
         const byte* in,
         word32      sz,
         byte*       digest);
 
-WOLFSSL_LOCAL int  tsip_Open();
+WOLFSSL_LOCAL int  tsip_Open(void);
 
-WOLFSSL_LOCAL void tsip_Close();
+WOLFSSL_LOCAL void tsip_Close(void);
 
-WOLFSSL_LOCAL int  tsip_hw_lock();
+#ifdef SINGLE_THREADED
+#define tsip_hw_lock() 0
+#define tsip_hw_unlock()
+#else
+WOLFSSL_LOCAL int  tsip_hw_lock(void);
+WOLFSSL_LOCAL void tsip_hw_unlock(void);
+#endif
 
-WOLFSSL_LOCAL void tsip_hw_unlock( void );
-
-WOLFSSL_LOCAL int  tsip_usable(const WOLFSSL *ssl,
+WOLFSSL_LOCAL int  tsip_usable(const struct WOLFSSL *ssl,
                                 uint8_t session_key_generated);
 
 WOLFSSL_LOCAL void tsip_inform_sflash_signedcacert(
@@ -613,37 +663,34 @@ WOLFSSL_LOCAL byte tsip_rootCAverified();
 
 WOLFSSL_LOCAL byte tsip_checkCA(word32 cmIdx);
 
-WOLFSSL_LOCAL int  wc_tsip_tls_RootCertVerify(
+WOLFSSL_API int wc_tsip_tls_RootCertVerify(
         const   byte* cert,   word32 cert_len,
         word32  key_n_start,  word32 key_n_len,
         word32  key_e_start,  word32 key_e_len,
         word32  cm_row);
 
-WOLFSSL_LOCAL int  wc_tsip_tls_CertVerify(
+WOLFSSL_API int wc_tsip_tls_CertVerify(
         const   uint8_t* cert,      uint32_t certSz,
         const   uint8_t* signature, uint32_t sigSz,
         uint32_t  key_n_start,      uint32_t key_n_len,
         uint32_t  key_e_start,      uint32_t key_e_len,
         uint8_t*  tsip_encRsaKeyIdx);
 
-WOLFSSL_LOCAL int  wc_tsip_generatePremasterSecret(
+WOLFSSL_API int wc_tsip_generatePremasterSecret(
         byte*   premaster,
         word32  preSz);
 
-WOLFSSL_LOCAL int  wc_tsip_generateSessionKey(
-        WOLFSSL* ssl,
+WOLFSSL_API int wc_tsip_generateSessionKey(
+        struct WOLFSSL* ssl,
         TsipUserCtx*    ctx,
         int             devId);
 
-WOLFSSL_LOCAL int wc_tsip_MakeRsaKey(int size, void* ctx);
-WOLFSSL_LOCAL int wc_tsip_RsaVerifyPkcs(wc_CryptoInfo* info,
-                                                TsipUserCtx* tuc);
+WOLFSSL_API int wc_tsip_MakeRsaKey(int size, void* ctx);
 
-WOLFSSL_LOCAL int  wc_tsip_GenerateRandBlock(byte* output, word32 size);
+WOLFSSL_API int wc_tsip_GenerateRandBlock(byte* output, word32 size);
 
 #if defined(WOLFSSL_RENESAS_TSIP_CRYPT_DEBUG)
 byte *ret2err(word32 ret);
-
 #endif
 
 #ifdef __cplusplus
