@@ -1,12 +1,12 @@
 /* wolfSSL-DTLS-PSK-Server.cs
  *
- * Copyright (C) 2006-2021 wolfSSL Inc.
+ * Copyright (C) 2006-2025 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
  * wolfSSL is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
- * the Free Software Foundation; either version 2 of the License, or
+ * the Free Software Foundation; either version 3 of the License, or
  * (at your option) any later version.
  *
  * wolfSSL is distributed in the hope that it will be useful,
@@ -21,6 +21,7 @@
 
 
 
+
 using System;
 
 using System.Runtime.InteropServices;
@@ -47,12 +48,12 @@ public class wolfSSL_DTLS_PSK_Server
     /// <returns>size of key set</returns>
     public static uint my_psk_server_cb(IntPtr ssl, string identity, IntPtr key, uint max_key)
     {
-        /* perform a check on the identity sent across 
+        /* perform a check on the identity sent across
          * log function must be set for print out of logging information
          */
         wolfssl.log(wolfssl.INFO_LOG, "PSK Client Identity = " + identity);
 
-        /* Use desired key, note must be a key smaller than max key size parameter 
+        /* Use desired key, note must be a key smaller than max key size parameter
             Replace this with desired key. Is trivial one for testing */
         if (max_key < 4)
             return 0;
@@ -77,9 +78,14 @@ public class wolfSSL_DTLS_PSK_Server
         IntPtr ssl;
 
         /* These paths should be changed according to use */
-        string fileCert = @"server-cert.pem";
-        string fileKey = @"server-key.pem";
-        StringBuilder dhparam = new StringBuilder("dh2048.pem");
+        string fileCert = wolfssl.setPath("server-cert.pem");
+        string fileKey = wolfssl.setPath("server-key.pem");
+        StringBuilder dhparam = new StringBuilder(wolfssl.setPath("dh2048.pem"));
+
+        if (fileCert == "" || fileKey == "" || dhparam.Length == 0) {
+            Console.WriteLine("Platform not supported");
+            return;
+        }
 
         wolfssl.psk_delegate psk_cb = new wolfssl.psk_delegate(my_psk_server_cb);
 
@@ -105,6 +111,12 @@ public class wolfSSL_DTLS_PSK_Server
             return;
         }
 
+        if (!File.Exists(dhparam.ToString())) {
+            Console.WriteLine("Could not find dh file");
+            wolfssl.CTX_free(ctx);
+            return;
+        }
+
 
         if (wolfssl.CTX_use_certificate_file(ctx, fileCert, wolfssl.SSL_FILETYPE_PEM) != wolfssl.SUCCESS)
         {
